import typing, clr, abc
from System.Buffers import OperationStatus
from System import ReadOnlySpan_1, Span_1, Range, Array_1, ArgumentException, Exception, ICloneable, ISpanFormattable, IUtf8SpanFormattable, IEquatable_1, IComparable_1, IComparable, Decimal, ReadOnlyMemory_1, IFormatProvider
from System.Collections import IDictionary
from System.Reflection import MethodBase
from System.IO import Stream
from System.Collections.Generic import IEnumerable_1, IEnumerator_1
from System.Globalization import UnicodeCategory, CultureInfo
from System.Runtime.Serialization import ISerializable

class Ascii(abc.ABC):
    @staticmethod
    def FromUtf16(source: ReadOnlySpan_1[str], destination: Span_1[int], bytesWritten: clr.Reference[int]) -> OperationStatus: ...
    @staticmethod
    def ToUtf16(source: ReadOnlySpan_1[int], destination: Span_1[str], charsWritten: clr.Reference[int]) -> OperationStatus: ...
    # Skipped Equals due to it being static, abstract and generic.

    Equals : Equals_MethodGroup
    class Equals_MethodGroup:
        @typing.overload
        def __call__(self, left: ReadOnlySpan_1[int], right: ReadOnlySpan_1[int]) -> bool:...
        @typing.overload
        def __call__(self, left: ReadOnlySpan_1[int], right: ReadOnlySpan_1[str]) -> bool:...
        @typing.overload
        def __call__(self, left: ReadOnlySpan_1[str], right: ReadOnlySpan_1[int]) -> bool:...
        @typing.overload
        def __call__(self, left: ReadOnlySpan_1[str], right: ReadOnlySpan_1[str]) -> bool:...

    # Skipped EqualsIgnoreCase due to it being static, abstract and generic.

    EqualsIgnoreCase : EqualsIgnoreCase_MethodGroup
    class EqualsIgnoreCase_MethodGroup:
        @typing.overload
        def __call__(self, left: ReadOnlySpan_1[int], right: ReadOnlySpan_1[int]) -> bool:...
        @typing.overload
        def __call__(self, left: ReadOnlySpan_1[int], right: ReadOnlySpan_1[str]) -> bool:...
        @typing.overload
        def __call__(self, left: ReadOnlySpan_1[str], right: ReadOnlySpan_1[int]) -> bool:...
        @typing.overload
        def __call__(self, left: ReadOnlySpan_1[str], right: ReadOnlySpan_1[str]) -> bool:...

    # Skipped IsValid due to it being static, abstract and generic.

    IsValid : IsValid_MethodGroup
    class IsValid_MethodGroup:
        @typing.overload
        def __call__(self, value: int) -> bool:...
        @typing.overload
        def __call__(self, value: str) -> bool:...
        @typing.overload
        def __call__(self, value: ReadOnlySpan_1[int]) -> bool:...
        @typing.overload
        def __call__(self, value: ReadOnlySpan_1[str]) -> bool:...

    # Skipped ToLower due to it being static, abstract and generic.

    ToLower : ToLower_MethodGroup
    class ToLower_MethodGroup:
        @typing.overload
        def __call__(self, source: ReadOnlySpan_1[int], destination: Span_1[int], bytesWritten: clr.Reference[int]) -> OperationStatus:...
        @typing.overload
        def __call__(self, source: ReadOnlySpan_1[str], destination: Span_1[int], bytesWritten: clr.Reference[int]) -> OperationStatus:...
        @typing.overload
        def __call__(self, source: ReadOnlySpan_1[str], destination: Span_1[str], charsWritten: clr.Reference[int]) -> OperationStatus:...
        @typing.overload
        def __call__(self, source: ReadOnlySpan_1[int], destination: Span_1[str], charsWritten: clr.Reference[int]) -> OperationStatus:...

    # Skipped ToLowerInPlace due to it being static, abstract and generic.

    ToLowerInPlace : ToLowerInPlace_MethodGroup
    class ToLowerInPlace_MethodGroup:
        @typing.overload
        def __call__(self, value: Span_1[int], bytesWritten: clr.Reference[int]) -> OperationStatus:...
        @typing.overload
        def __call__(self, value: Span_1[str], charsWritten: clr.Reference[int]) -> OperationStatus:...

    # Skipped ToUpper due to it being static, abstract and generic.

    ToUpper : ToUpper_MethodGroup
    class ToUpper_MethodGroup:
        @typing.overload
        def __call__(self, source: ReadOnlySpan_1[int], destination: Span_1[int], bytesWritten: clr.Reference[int]) -> OperationStatus:...
        @typing.overload
        def __call__(self, source: ReadOnlySpan_1[str], destination: Span_1[int], bytesWritten: clr.Reference[int]) -> OperationStatus:...
        @typing.overload
        def __call__(self, source: ReadOnlySpan_1[str], destination: Span_1[str], charsWritten: clr.Reference[int]) -> OperationStatus:...
        @typing.overload
        def __call__(self, source: ReadOnlySpan_1[int], destination: Span_1[str], charsWritten: clr.Reference[int]) -> OperationStatus:...

    # Skipped ToUpperInPlace due to it being static, abstract and generic.

    ToUpperInPlace : ToUpperInPlace_MethodGroup
    class ToUpperInPlace_MethodGroup:
        @typing.overload
        def __call__(self, value: Span_1[int], bytesWritten: clr.Reference[int]) -> OperationStatus:...
        @typing.overload
        def __call__(self, value: Span_1[str], charsWritten: clr.Reference[int]) -> OperationStatus:...

    # Skipped Trim due to it being static, abstract and generic.

    Trim : Trim_MethodGroup
    class Trim_MethodGroup:
        @typing.overload
        def __call__(self, value: ReadOnlySpan_1[int]) -> Range:...
        @typing.overload
        def __call__(self, value: ReadOnlySpan_1[str]) -> Range:...

    # Skipped TrimEnd due to it being static, abstract and generic.

    TrimEnd : TrimEnd_MethodGroup
    class TrimEnd_MethodGroup:
        @typing.overload
        def __call__(self, value: ReadOnlySpan_1[int]) -> Range:...
        @typing.overload
        def __call__(self, value: ReadOnlySpan_1[str]) -> Range:...

    # Skipped TrimStart due to it being static, abstract and generic.

    TrimStart : TrimStart_MethodGroup
    class TrimStart_MethodGroup:
        @typing.overload
        def __call__(self, value: ReadOnlySpan_1[int]) -> Range:...
        @typing.overload
        def __call__(self, value: ReadOnlySpan_1[str]) -> Range:...



class ASCIIEncoding(Encoding):
    def __init__(self) -> None: ...
    @property
    def BodyName(self) -> str: ...
    @property
    def CodePage(self) -> int: ...
    @property
    def DecoderFallback(self) -> DecoderFallback: ...
    @DecoderFallback.setter
    def DecoderFallback(self, value: DecoderFallback) -> DecoderFallback: ...
    @property
    def EncoderFallback(self) -> EncoderFallback: ...
    @EncoderFallback.setter
    def EncoderFallback(self, value: EncoderFallback) -> EncoderFallback: ...
    @property
    def EncodingName(self) -> str: ...
    @property
    def HeaderName(self) -> str: ...
    @property
    def IsBrowserDisplay(self) -> bool: ...
    @property
    def IsBrowserSave(self) -> bool: ...
    @property
    def IsMailNewsDisplay(self) -> bool: ...
    @property
    def IsMailNewsSave(self) -> bool: ...
    @property
    def IsReadOnly(self) -> bool: ...
    @IsReadOnly.setter
    def IsReadOnly(self, value: bool) -> bool: ...
    @property
    def IsSingleByte(self) -> bool: ...
    @property
    def Preamble(self) -> ReadOnlySpan_1[int]: ...
    @property
    def WebName(self) -> str: ...
    @property
    def WindowsCodePage(self) -> int: ...
    def GetDecoder(self) -> Decoder: ...
    def GetEncoder(self) -> Encoder: ...
    def GetMaxByteCount(self, charCount: int) -> int: ...
    def GetMaxCharCount(self, byteCount: int) -> int: ...
    def GetString(self, bytes: Array_1[int], byteIndex: int, byteCount: int) -> str: ...
    def TryGetBytes(self, chars: ReadOnlySpan_1[str], bytes: Span_1[int], bytesWritten: clr.Reference[int]) -> bool: ...
    def TryGetChars(self, bytes: ReadOnlySpan_1[int], chars: Span_1[str], charsWritten: clr.Reference[int]) -> bool: ...
    # Skipped GetByteCount due to it being static, abstract and generic.

    GetByteCount : GetByteCount_MethodGroup
    class GetByteCount_MethodGroup:
        @typing.overload
        def __call__(self, chars: ReadOnlySpan_1[str]) -> int:...
        @typing.overload
        def __call__(self, chars: str) -> int:...
        @typing.overload
        def __call__(self, chars: clr.Reference[str], count: int) -> int:...
        @typing.overload
        def __call__(self, chars: Array_1[str], index: int, count: int) -> int:...

    # Skipped GetBytes due to it being static, abstract and generic.

    GetBytes : GetBytes_MethodGroup
    class GetBytes_MethodGroup:
        @typing.overload
        def __call__(self, chars: ReadOnlySpan_1[str], bytes: Span_1[int]) -> int:...
        @typing.overload
        def __call__(self, chars: clr.Reference[str], charCount: int, bytes: clr.Reference[int], byteCount: int) -> int:...
        @typing.overload
        def __call__(self, chars: Array_1[str], charIndex: int, charCount: int, bytes: Array_1[int], byteIndex: int) -> int:...
        @typing.overload
        def __call__(self, chars: str, charIndex: int, charCount: int, bytes: Array_1[int], byteIndex: int) -> int:...

    # Skipped GetCharCount due to it being static, abstract and generic.

    GetCharCount : GetCharCount_MethodGroup
    class GetCharCount_MethodGroup:
        @typing.overload
        def __call__(self, bytes: ReadOnlySpan_1[int]) -> int:...
        @typing.overload
        def __call__(self, bytes: clr.Reference[int], count: int) -> int:...
        @typing.overload
        def __call__(self, bytes: Array_1[int], index: int, count: int) -> int:...

    # Skipped GetChars due to it being static, abstract and generic.

    GetChars : GetChars_MethodGroup
    class GetChars_MethodGroup:
        @typing.overload
        def __call__(self, bytes: ReadOnlySpan_1[int], chars: Span_1[str]) -> int:...
        @typing.overload
        def __call__(self, bytes: clr.Reference[int], byteCount: int, chars: clr.Reference[str], charCount: int) -> int:...
        @typing.overload
        def __call__(self, bytes: Array_1[int], byteIndex: int, byteCount: int, chars: Array_1[str], charIndex: int) -> int:...



class CompositeFormat:
    @property
    def Format(self) -> str: ...
    @property
    def MinimumArgumentCount(self) -> int: ...
    @staticmethod
    def Parse(format: str) -> CompositeFormat: ...


class Decoder(abc.ABC):
    @property
    def Fallback(self) -> DecoderFallback: ...
    @Fallback.setter
    def Fallback(self, value: DecoderFallback) -> DecoderFallback: ...
    @property
    def FallbackBuffer(self) -> DecoderFallbackBuffer: ...
    def Reset(self) -> None: ...
    # Skipped Convert due to it being static, abstract and generic.

    Convert : Convert_MethodGroup
    class Convert_MethodGroup:
        @typing.overload
        def __call__(self, bytes: ReadOnlySpan_1[int], chars: Span_1[str], flush: bool, bytesUsed: clr.Reference[int], charsUsed: clr.Reference[int], completed: clr.Reference[bool]) -> None:...
        @typing.overload
        def __call__(self, bytes: clr.Reference[int], byteCount: int, chars: clr.Reference[str], charCount: int, flush: bool, bytesUsed: clr.Reference[int], charsUsed: clr.Reference[int], completed: clr.Reference[bool]) -> None:...
        @typing.overload
        def __call__(self, bytes: Array_1[int], byteIndex: int, byteCount: int, chars: Array_1[str], charIndex: int, charCount: int, flush: bool, bytesUsed: clr.Reference[int], charsUsed: clr.Reference[int], completed: clr.Reference[bool]) -> None:...

    # Skipped GetCharCount due to it being static, abstract and generic.

    GetCharCount : GetCharCount_MethodGroup
    class GetCharCount_MethodGroup:
        @typing.overload
        def __call__(self, bytes: ReadOnlySpan_1[int], flush: bool) -> int:...
        @typing.overload
        def __call__(self, bytes: Array_1[int], index: int, count: int) -> int:...
        @typing.overload
        def __call__(self, bytes: clr.Reference[int], count: int, flush: bool) -> int:...
        @typing.overload
        def __call__(self, bytes: Array_1[int], index: int, count: int, flush: bool) -> int:...

    # Skipped GetChars due to it being static, abstract and generic.

    GetChars : GetChars_MethodGroup
    class GetChars_MethodGroup:
        @typing.overload
        def __call__(self, bytes: ReadOnlySpan_1[int], chars: Span_1[str], flush: bool) -> int:...
        @typing.overload
        def __call__(self, bytes: Array_1[int], byteIndex: int, byteCount: int, chars: Array_1[str], charIndex: int) -> int:...
        @typing.overload
        def __call__(self, bytes: clr.Reference[int], byteCount: int, chars: clr.Reference[str], charCount: int, flush: bool) -> int:...
        @typing.overload
        def __call__(self, bytes: Array_1[int], byteIndex: int, byteCount: int, chars: Array_1[str], charIndex: int, flush: bool) -> int:...



class DecoderExceptionFallback(DecoderFallback):
    def __init__(self) -> None: ...
    @property
    def MaxCharCount(self) -> int: ...
    def CreateFallbackBuffer(self) -> DecoderFallbackBuffer: ...
    def Equals(self, value: typing.Any) -> bool: ...
    def GetHashCode(self) -> int: ...


class DecoderExceptionFallbackBuffer(DecoderFallbackBuffer):
    def __init__(self) -> None: ...
    @property
    def Remaining(self) -> int: ...
    def Fallback(self, bytesUnknown: Array_1[int], index: int) -> bool: ...
    def GetNextChar(self) -> str: ...
    def MovePrevious(self) -> bool: ...


class DecoderFallback(abc.ABC):
    @classmethod
    @property
    def ExceptionFallback(cls) -> DecoderFallback: ...
    @property
    def MaxCharCount(self) -> int: ...
    @classmethod
    @property
    def ReplacementFallback(cls) -> DecoderFallback: ...
    @abc.abstractmethod
    def CreateFallbackBuffer(self) -> DecoderFallbackBuffer: ...


class DecoderFallbackBuffer(abc.ABC):
    @property
    def Remaining(self) -> int: ...
    @abc.abstractmethod
    def Fallback(self, bytesUnknown: Array_1[int], index: int) -> bool: ...
    @abc.abstractmethod
    def GetNextChar(self) -> str: ...
    @abc.abstractmethod
    def MovePrevious(self) -> bool: ...
    def Reset(self) -> None: ...


class DecoderFallbackException(ArgumentException):
    @typing.overload
    def __init__(self) -> None: ...
    @typing.overload
    def __init__(self, message: str) -> None: ...
    @typing.overload
    def __init__(self, message: str, bytesUnknown: Array_1[int], index: int) -> None: ...
    @typing.overload
    def __init__(self, message: str, innerException: Exception) -> None: ...
    @property
    def BytesUnknown(self) -> Array_1[int]: ...
    @property
    def Data(self) -> IDictionary: ...
    @property
    def HelpLink(self) -> str: ...
    @HelpLink.setter
    def HelpLink(self, value: str) -> str: ...
    @property
    def HResult(self) -> int: ...
    @HResult.setter
    def HResult(self, value: int) -> int: ...
    @property
    def Index(self) -> int: ...
    @property
    def InnerException(self) -> Exception: ...
    @property
    def Message(self) -> str: ...
    @property
    def ParamName(self) -> str: ...
    @property
    def Source(self) -> str: ...
    @Source.setter
    def Source(self, value: str) -> str: ...
    @property
    def StackTrace(self) -> str: ...
    @property
    def TargetSite(self) -> MethodBase: ...


class DecoderReplacementFallback(DecoderFallback):
    @typing.overload
    def __init__(self) -> None: ...
    @typing.overload
    def __init__(self, replacement: str) -> None: ...
    @property
    def DefaultString(self) -> str: ...
    @property
    def MaxCharCount(self) -> int: ...
    def CreateFallbackBuffer(self) -> DecoderFallbackBuffer: ...
    def Equals(self, value: typing.Any) -> bool: ...
    def GetHashCode(self) -> int: ...


class DecoderReplacementFallbackBuffer(DecoderFallbackBuffer):
    def __init__(self, fallback: DecoderReplacementFallback) -> None: ...
    @property
    def Remaining(self) -> int: ...
    def Fallback(self, bytesUnknown: Array_1[int], index: int) -> bool: ...
    def GetNextChar(self) -> str: ...
    def MovePrevious(self) -> bool: ...
    def Reset(self) -> None: ...


class Encoder(abc.ABC):
    @property
    def Fallback(self) -> EncoderFallback: ...
    @Fallback.setter
    def Fallback(self, value: EncoderFallback) -> EncoderFallback: ...
    @property
    def FallbackBuffer(self) -> EncoderFallbackBuffer: ...
    def Reset(self) -> None: ...
    # Skipped Convert due to it being static, abstract and generic.

    Convert : Convert_MethodGroup
    class Convert_MethodGroup:
        @typing.overload
        def __call__(self, chars: ReadOnlySpan_1[str], bytes: Span_1[int], flush: bool, charsUsed: clr.Reference[int], bytesUsed: clr.Reference[int], completed: clr.Reference[bool]) -> None:...
        @typing.overload
        def __call__(self, chars: clr.Reference[str], charCount: int, bytes: clr.Reference[int], byteCount: int, flush: bool, charsUsed: clr.Reference[int], bytesUsed: clr.Reference[int], completed: clr.Reference[bool]) -> None:...
        @typing.overload
        def __call__(self, chars: Array_1[str], charIndex: int, charCount: int, bytes: Array_1[int], byteIndex: int, byteCount: int, flush: bool, charsUsed: clr.Reference[int], bytesUsed: clr.Reference[int], completed: clr.Reference[bool]) -> None:...

    # Skipped GetByteCount due to it being static, abstract and generic.

    GetByteCount : GetByteCount_MethodGroup
    class GetByteCount_MethodGroup:
        @typing.overload
        def __call__(self, chars: ReadOnlySpan_1[str], flush: bool) -> int:...
        @typing.overload
        def __call__(self, chars: clr.Reference[str], count: int, flush: bool) -> int:...
        @typing.overload
        def __call__(self, chars: Array_1[str], index: int, count: int, flush: bool) -> int:...

    # Skipped GetBytes due to it being static, abstract and generic.

    GetBytes : GetBytes_MethodGroup
    class GetBytes_MethodGroup:
        @typing.overload
        def __call__(self, chars: ReadOnlySpan_1[str], bytes: Span_1[int], flush: bool) -> int:...
        @typing.overload
        def __call__(self, chars: clr.Reference[str], charCount: int, bytes: clr.Reference[int], byteCount: int, flush: bool) -> int:...
        @typing.overload
        def __call__(self, chars: Array_1[str], charIndex: int, charCount: int, bytes: Array_1[int], byteIndex: int, flush: bool) -> int:...



class EncoderExceptionFallback(EncoderFallback):
    def __init__(self) -> None: ...
    @property
    def MaxCharCount(self) -> int: ...
    def CreateFallbackBuffer(self) -> EncoderFallbackBuffer: ...
    def Equals(self, value: typing.Any) -> bool: ...
    def GetHashCode(self) -> int: ...


class EncoderExceptionFallbackBuffer(EncoderFallbackBuffer):
    def __init__(self) -> None: ...
    @property
    def Remaining(self) -> int: ...
    def GetNextChar(self) -> str: ...
    def MovePrevious(self) -> bool: ...
    # Skipped Fallback due to it being static, abstract and generic.

    Fallback : Fallback_MethodGroup
    class Fallback_MethodGroup:
        @typing.overload
        def __call__(self, charUnknown: str, index: int) -> bool:...
        @typing.overload
        def __call__(self, charUnknownHigh: str, charUnknownLow: str, index: int) -> bool:...



class EncoderFallback(abc.ABC):
    @classmethod
    @property
    def ExceptionFallback(cls) -> EncoderFallback: ...
    @property
    def MaxCharCount(self) -> int: ...
    @classmethod
    @property
    def ReplacementFallback(cls) -> EncoderFallback: ...
    @abc.abstractmethod
    def CreateFallbackBuffer(self) -> EncoderFallbackBuffer: ...


class EncoderFallbackBuffer(abc.ABC):
    @property
    def Remaining(self) -> int: ...
    @abc.abstractmethod
    def GetNextChar(self) -> str: ...
    @abc.abstractmethod
    def MovePrevious(self) -> bool: ...
    def Reset(self) -> None: ...
    # Skipped Fallback due to it being static, abstract and generic.

    Fallback : Fallback_MethodGroup
    class Fallback_MethodGroup:
        @typing.overload
        def __call__(self, charUnknown: str, index: int) -> bool:...
        @typing.overload
        def __call__(self, charUnknownHigh: str, charUnknownLow: str, index: int) -> bool:...



class EncoderFallbackException(ArgumentException):
    @typing.overload
    def __init__(self) -> None: ...
    @typing.overload
    def __init__(self, message: str) -> None: ...
    @typing.overload
    def __init__(self, message: str, innerException: Exception) -> None: ...
    @property
    def CharUnknown(self) -> str: ...
    @property
    def CharUnknownHigh(self) -> str: ...
    @property
    def CharUnknownLow(self) -> str: ...
    @property
    def Data(self) -> IDictionary: ...
    @property
    def HelpLink(self) -> str: ...
    @HelpLink.setter
    def HelpLink(self, value: str) -> str: ...
    @property
    def HResult(self) -> int: ...
    @HResult.setter
    def HResult(self, value: int) -> int: ...
    @property
    def Index(self) -> int: ...
    @property
    def InnerException(self) -> Exception: ...
    @property
    def Message(self) -> str: ...
    @property
    def ParamName(self) -> str: ...
    @property
    def Source(self) -> str: ...
    @Source.setter
    def Source(self, value: str) -> str: ...
    @property
    def StackTrace(self) -> str: ...
    @property
    def TargetSite(self) -> MethodBase: ...
    def IsUnknownSurrogate(self) -> bool: ...


class EncoderReplacementFallback(EncoderFallback):
    @typing.overload
    def __init__(self) -> None: ...
    @typing.overload
    def __init__(self, replacement: str) -> None: ...
    @property
    def DefaultString(self) -> str: ...
    @property
    def MaxCharCount(self) -> int: ...
    def CreateFallbackBuffer(self) -> EncoderFallbackBuffer: ...
    def Equals(self, value: typing.Any) -> bool: ...
    def GetHashCode(self) -> int: ...


class EncoderReplacementFallbackBuffer(EncoderFallbackBuffer):
    def __init__(self, fallback: EncoderReplacementFallback) -> None: ...
    @property
    def Remaining(self) -> int: ...
    def GetNextChar(self) -> str: ...
    def MovePrevious(self) -> bool: ...
    def Reset(self) -> None: ...
    # Skipped Fallback due to it being static, abstract and generic.

    Fallback : Fallback_MethodGroup
    class Fallback_MethodGroup:
        @typing.overload
        def __call__(self, charUnknown: str, index: int) -> bool:...
        @typing.overload
        def __call__(self, charUnknownHigh: str, charUnknownLow: str, index: int) -> bool:...



class Encoding(ICloneable, abc.ABC):
    @classmethod
    @property
    def ASCII(cls) -> Encoding: ...
    @classmethod
    @property
    def BigEndianUnicode(cls) -> Encoding: ...
    @property
    def BodyName(self) -> str: ...
    @property
    def CodePage(self) -> int: ...
    @property
    def DecoderFallback(self) -> DecoderFallback: ...
    @DecoderFallback.setter
    def DecoderFallback(self, value: DecoderFallback) -> DecoderFallback: ...
    @classmethod
    @property
    def Default(cls) -> Encoding: ...
    @property
    def EncoderFallback(self) -> EncoderFallback: ...
    @EncoderFallback.setter
    def EncoderFallback(self, value: EncoderFallback) -> EncoderFallback: ...
    @property
    def EncodingName(self) -> str: ...
    @property
    def HeaderName(self) -> str: ...
    @property
    def IsBrowserDisplay(self) -> bool: ...
    @property
    def IsBrowserSave(self) -> bool: ...
    @property
    def IsMailNewsDisplay(self) -> bool: ...
    @property
    def IsMailNewsSave(self) -> bool: ...
    @property
    def IsReadOnly(self) -> bool: ...
    @IsReadOnly.setter
    def IsReadOnly(self, value: bool) -> bool: ...
    @property
    def IsSingleByte(self) -> bool: ...
    @classmethod
    @property
    def Latin1(cls) -> Encoding: ...
    @property
    def Preamble(self) -> ReadOnlySpan_1[int]: ...
    @classmethod
    @property
    def Unicode(cls) -> Encoding: ...
    @classmethod
    @property
    def UTF32(cls) -> Encoding: ...
    @classmethod
    @property
    def UTF7(cls) -> Encoding: ...
    @classmethod
    @property
    def UTF8(cls) -> Encoding: ...
    @property
    def WebName(self) -> str: ...
    @property
    def WindowsCodePage(self) -> int: ...
    def Clone(self) -> typing.Any: ...
    @staticmethod
    def CreateTranscodingStream(innerStream: Stream, innerStreamEncoding: Encoding, outerStreamEncoding: Encoding, leaveOpen: bool = ...) -> Stream: ...
    def Equals(self, value: typing.Any) -> bool: ...
    def GetDecoder(self) -> Decoder: ...
    def GetEncoder(self) -> Encoder: ...
    @staticmethod
    def GetEncodings() -> Array_1[EncodingInfo]: ...
    def GetHashCode(self) -> int: ...
    @abc.abstractmethod
    def GetMaxByteCount(self, charCount: int) -> int: ...
    @abc.abstractmethod
    def GetMaxCharCount(self, byteCount: int) -> int: ...
    def GetPreamble(self) -> Array_1[int]: ...
    @staticmethod
    def RegisterProvider(provider: EncodingProvider) -> None: ...
    def TryGetBytes(self, chars: ReadOnlySpan_1[str], bytes: Span_1[int], bytesWritten: clr.Reference[int]) -> bool: ...
    def TryGetChars(self, bytes: ReadOnlySpan_1[int], chars: Span_1[str], charsWritten: clr.Reference[int]) -> bool: ...
    # Skipped Convert due to it being static, abstract and generic.

    Convert : Convert_MethodGroup
    class Convert_MethodGroup:
        @typing.overload
        def __call__(self, srcEncoding: Encoding, dstEncoding: Encoding, bytes: Array_1[int]) -> Array_1[int]:...
        @typing.overload
        def __call__(self, srcEncoding: Encoding, dstEncoding: Encoding, bytes: Array_1[int], index: int, count: int) -> Array_1[int]:...

    # Skipped GetByteCount due to it being static, abstract and generic.

    GetByteCount : GetByteCount_MethodGroup
    class GetByteCount_MethodGroup:
        @typing.overload
        def __call__(self, chars: Array_1[str]) -> int:...
        @typing.overload
        def __call__(self, chars: ReadOnlySpan_1[str]) -> int:...
        @typing.overload
        def __call__(self, s: str) -> int:...
        @typing.overload
        def __call__(self, chars: clr.Reference[str], count: int) -> int:...
        @typing.overload
        def __call__(self, chars: Array_1[str], index: int, count: int) -> int:...
        @typing.overload
        def __call__(self, s: str, index: int, count: int) -> int:...

    # Skipped GetBytes due to it being static, abstract and generic.

    GetBytes : GetBytes_MethodGroup
    class GetBytes_MethodGroup:
        @typing.overload
        def __call__(self, chars: Array_1[str]) -> Array_1[int]:...
        @typing.overload
        def __call__(self, s: str) -> Array_1[int]:...
        @typing.overload
        def __call__(self, chars: ReadOnlySpan_1[str], bytes: Span_1[int]) -> int:...
        @typing.overload
        def __call__(self, chars: Array_1[str], index: int, count: int) -> Array_1[int]:...
        @typing.overload
        def __call__(self, s: str, index: int, count: int) -> Array_1[int]:...
        @typing.overload
        def __call__(self, chars: clr.Reference[str], charCount: int, bytes: clr.Reference[int], byteCount: int) -> int:...
        @typing.overload
        def __call__(self, chars: Array_1[str], charIndex: int, charCount: int, bytes: Array_1[int], byteIndex: int) -> int:...
        @typing.overload
        def __call__(self, s: str, charIndex: int, charCount: int, bytes: Array_1[int], byteIndex: int) -> int:...

    # Skipped GetCharCount due to it being static, abstract and generic.

    GetCharCount : GetCharCount_MethodGroup
    class GetCharCount_MethodGroup:
        @typing.overload
        def __call__(self, bytes: Array_1[int]) -> int:...
        @typing.overload
        def __call__(self, bytes: ReadOnlySpan_1[int]) -> int:...
        @typing.overload
        def __call__(self, bytes: clr.Reference[int], count: int) -> int:...
        @typing.overload
        def __call__(self, bytes: Array_1[int], index: int, count: int) -> int:...

    # Skipped GetChars due to it being static, abstract and generic.

    GetChars : GetChars_MethodGroup
    class GetChars_MethodGroup:
        @typing.overload
        def __call__(self, bytes: Array_1[int]) -> Array_1[str]:...
        @typing.overload
        def __call__(self, bytes: ReadOnlySpan_1[int], chars: Span_1[str]) -> int:...
        @typing.overload
        def __call__(self, bytes: Array_1[int], index: int, count: int) -> Array_1[str]:...
        @typing.overload
        def __call__(self, bytes: clr.Reference[int], byteCount: int, chars: clr.Reference[str], charCount: int) -> int:...
        @typing.overload
        def __call__(self, bytes: Array_1[int], byteIndex: int, byteCount: int, chars: Array_1[str], charIndex: int) -> int:...

    # Skipped GetEncoding due to it being static, abstract and generic.

    GetEncoding : GetEncoding_MethodGroup
    class GetEncoding_MethodGroup:
        @typing.overload
        def __call__(self, codepage: int) -> Encoding:...
        @typing.overload
        def __call__(self, name: str) -> Encoding:...
        @typing.overload
        def __call__(self, codepage: int, encoderFallback: EncoderFallback, decoderFallback: DecoderFallback) -> Encoding:...
        @typing.overload
        def __call__(self, name: str, encoderFallback: EncoderFallback, decoderFallback: DecoderFallback) -> Encoding:...

    # Skipped GetString due to it being static, abstract and generic.

    GetString : GetString_MethodGroup
    class GetString_MethodGroup:
        @typing.overload
        def __call__(self, bytes: Array_1[int]) -> str:...
        @typing.overload
        def __call__(self, bytes: ReadOnlySpan_1[int]) -> str:...
        @typing.overload
        def __call__(self, bytes: clr.Reference[int], byteCount: int) -> str:...
        @typing.overload
        def __call__(self, bytes: Array_1[int], index: int, count: int) -> str:...

    # Skipped IsAlwaysNormalized due to it being static, abstract and generic.

    IsAlwaysNormalized : IsAlwaysNormalized_MethodGroup
    class IsAlwaysNormalized_MethodGroup:
        @typing.overload
        def __call__(self) -> bool:...
        @typing.overload
        def __call__(self, form: NormalizationForm) -> bool:...



class EncodingInfo:
    def __init__(self, provider: EncodingProvider, codePage: int, name: str, displayName: str) -> None: ...
    @property
    def CodePage(self) -> int: ...
    @property
    def DisplayName(self) -> str: ...
    @property
    def Name(self) -> str: ...
    def Equals(self, value: typing.Any) -> bool: ...
    def GetEncoding(self) -> Encoding: ...
    def GetHashCode(self) -> int: ...


class EncodingProvider(abc.ABC):
    def __init__(self) -> None: ...
    def GetEncodings(self) -> IEnumerable_1[EncodingInfo]: ...
    # Skipped GetEncoding due to it being static, abstract and generic.

    GetEncoding : GetEncoding_MethodGroup
    class GetEncoding_MethodGroup:
        @typing.overload
        def __call__(self, codepage: int) -> Encoding:...
        @typing.overload
        def __call__(self, name: str) -> Encoding:...
        @typing.overload
        def __call__(self, codepage: int, encoderFallback: EncoderFallback, decoderFallback: DecoderFallback) -> Encoding:...
        @typing.overload
        def __call__(self, name: str, encoderFallback: EncoderFallback, decoderFallback: DecoderFallback) -> Encoding:...



class NormalizationForm(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    FormC : NormalizationForm # 1
    FormD : NormalizationForm # 2
    FormKC : NormalizationForm # 5
    FormKD : NormalizationForm # 6


class Rune(ISpanFormattable, IUtf8SpanFormattable, IEquatable_1[Rune], IComparable_1[Rune], IComparable_0):
    # Constructor .ctor(value : UInt32) was skipped since it collides with above method
    @typing.overload
    def __init__(self, ch: str) -> None: ...
    @typing.overload
    def __init__(self, highSurrogate: str, lowSurrogate: str) -> None: ...
    @typing.overload
    def __init__(self, value: int) -> None: ...
    @property
    def IsAscii(self) -> bool: ...
    @property
    def IsBmp(self) -> bool: ...
    @property
    def Plane(self) -> int: ...
    @classmethod
    @property
    def ReplacementChar(cls) -> Rune: ...
    @property
    def Utf16SequenceLength(self) -> int: ...
    @property
    def Utf8SequenceLength(self) -> int: ...
    @property
    def Value(self) -> int: ...
    def CompareTo(self, other: Rune) -> int: ...
    @staticmethod
    def DecodeFromUtf16(source: ReadOnlySpan_1[str], result: clr.Reference[Rune], charsConsumed: clr.Reference[int]) -> OperationStatus: ...
    @staticmethod
    def DecodeFromUtf8(source: ReadOnlySpan_1[int], result: clr.Reference[Rune], bytesConsumed: clr.Reference[int]) -> OperationStatus: ...
    @staticmethod
    def DecodeLastFromUtf16(source: ReadOnlySpan_1[str], result: clr.Reference[Rune], charsConsumed: clr.Reference[int]) -> OperationStatus: ...
    @staticmethod
    def DecodeLastFromUtf8(source: ReadOnlySpan_1[int], value: clr.Reference[Rune], bytesConsumed: clr.Reference[int]) -> OperationStatus: ...
    def EncodeToUtf16(self, destination: Span_1[str]) -> int: ...
    def EncodeToUtf8(self, destination: Span_1[int]) -> int: ...
    def GetHashCode(self) -> int: ...
    @staticmethod
    def GetNumericValue(value: Rune) -> float: ...
    @staticmethod
    def GetRuneAt(input: str, index: int) -> Rune: ...
    @staticmethod
    def GetUnicodeCategory(value: Rune) -> UnicodeCategory: ...
    @staticmethod
    def IsControl(value: Rune) -> bool: ...
    @staticmethod
    def IsDigit(value: Rune) -> bool: ...
    @staticmethod
    def IsLetter(value: Rune) -> bool: ...
    @staticmethod
    def IsLetterOrDigit(value: Rune) -> bool: ...
    @staticmethod
    def IsLower(value: Rune) -> bool: ...
    @staticmethod
    def IsNumber(value: Rune) -> bool: ...
    @staticmethod
    def IsPunctuation(value: Rune) -> bool: ...
    @staticmethod
    def IsSeparator(value: Rune) -> bool: ...
    @staticmethod
    def IsSymbol(value: Rune) -> bool: ...
    @staticmethod
    def IsUpper(value: Rune) -> bool: ...
    @staticmethod
    def IsWhiteSpace(value: Rune) -> bool: ...
    def __eq__(self, left: Rune, right: Rune) -> bool: ...
    # Operator not supported op_Explicit(ch: Char)
    # Operator not supported op_Explicit(value: UInt32)
    # Operator not supported op_Explicit(value: Int32)
    def __gt__(self, left: Rune, right: Rune) -> bool: ...
    def __ge__(self, left: Rune, right: Rune) -> bool: ...
    def __ne__(self, left: Rune, right: Rune) -> bool: ...
    def __lt__(self, left: Rune, right: Rune) -> bool: ...
    def __le__(self, left: Rune, right: Rune) -> bool: ...
    @staticmethod
    def ToLower(value: Rune, culture: CultureInfo) -> Rune: ...
    @staticmethod
    def ToLowerInvariant(value: Rune) -> Rune: ...
    def ToString(self) -> str: ...
    @staticmethod
    def ToUpper(value: Rune, culture: CultureInfo) -> Rune: ...
    @staticmethod
    def ToUpperInvariant(value: Rune) -> Rune: ...
    def TryEncodeToUtf16(self, destination: Span_1[str], charsWritten: clr.Reference[int]) -> bool: ...
    def TryEncodeToUtf8(self, destination: Span_1[int], bytesWritten: clr.Reference[int]) -> bool: ...
    @staticmethod
    def TryGetRuneAt(input: str, index: int, value: clr.Reference[Rune]) -> bool: ...
    # Skipped Equals due to it being static, abstract and generic.

    Equals : Equals_MethodGroup
    class Equals_MethodGroup:
        @typing.overload
        def __call__(self, other: Rune) -> bool:...
        @typing.overload
        def __call__(self, obj: typing.Any) -> bool:...

    # Skipped IsValid due to it being static, abstract and generic.

    IsValid : IsValid_MethodGroup
    class IsValid_MethodGroup:
        def __call__(self, value: int) -> bool:...
        # Method IsValid(value : UInt32) was skipped since it collides with above method

    # Skipped TryCreate due to it being static, abstract and generic.

    TryCreate : TryCreate_MethodGroup
    class TryCreate_MethodGroup:
        @typing.overload
        def __call__(self, ch: str, result: clr.Reference[Rune]) -> bool:...
        @typing.overload
        def __call__(self, value: int, result: clr.Reference[Rune]) -> bool:...
        # Method TryCreate(value : UInt32, result : Rune&) was skipped since it collides with above method
        @typing.overload
        def __call__(self, highSurrogate: str, lowSurrogate: str, result: clr.Reference[Rune]) -> bool:...



class SpanLineEnumerator:
    @property
    def Current(self) -> ReadOnlySpan_1[str]: ...
    def GetEnumerator(self) -> SpanLineEnumerator: ...
    def MoveNext(self) -> bool: ...


class SpanRuneEnumerator:
    @property
    def Current(self) -> Rune: ...
    def GetEnumerator(self) -> SpanRuneEnumerator: ...
    def MoveNext(self) -> bool: ...


class StringBuilder(ISerializable):
    @typing.overload
    def __init__(self) -> None: ...
    @typing.overload
    def __init__(self, capacity: int) -> None: ...
    @typing.overload
    def __init__(self, capacity: int, maxCapacity: int) -> None: ...
    @typing.overload
    def __init__(self, value: str) -> None: ...
    @typing.overload
    def __init__(self, value: str, capacity: int) -> None: ...
    @typing.overload
    def __init__(self, value: str, startIndex: int, length: int, capacity: int) -> None: ...
    @property
    def Capacity(self) -> int: ...
    @Capacity.setter
    def Capacity(self, value: int) -> int: ...
    @property
    def Chars(self) -> str: ...
    @Chars.setter
    def Chars(self, value: str) -> str: ...
    @property
    def Length(self) -> int: ...
    @Length.setter
    def Length(self, value: int) -> int: ...
    @property
    def MaxCapacity(self) -> int: ...
    def Clear(self) -> StringBuilder: ...
    def EnsureCapacity(self, capacity: int) -> int: ...
    def GetChunks(self) -> StringBuilder.ChunkEnumerator: ...
    def Remove(self, startIndex: int, length: int) -> StringBuilder: ...
    # Skipped Append due to it being static, abstract and generic.

    Append : Append_MethodGroup
    class Append_MethodGroup:
        @typing.overload
        def __call__(self, value: float) -> StringBuilder:...
        # Method Append(value : Double) was skipped since it collides with above method
        @typing.overload
        def __call__(self, value: str) -> StringBuilder:...
        # Method Append(value : SByte) was skipped since it collides with above method
        # Method Append(value : Byte) was skipped since it collides with above method
        # Method Append(value : Int16) was skipped since it collides with above method
        # Method Append(value : Int32) was skipped since it collides with above method
        # Method Append(value : Int64) was skipped since it collides with above method
        # Method Append(value : UInt16) was skipped since it collides with above method
        # Method Append(value : UInt32) was skipped since it collides with above method
        # Method Append(value : UInt64) was skipped since it collides with above method
        @typing.overload
        def __call__(self, value: Decimal) -> StringBuilder:...
        @typing.overload
        def __call__(self, value: Array_1[str]) -> StringBuilder:...
        @typing.overload
        def __call__(self, value: ReadOnlySpan_1[str]) -> StringBuilder:...
        @typing.overload
        def __call__(self, value: ReadOnlyMemory_1[str]) -> StringBuilder:...
        # Method Append(value : String) was skipped since it collides with above method
        # Method Append(value : Boolean) was skipped since it collides with above method
        @typing.overload
        def __call__(self, value: StringBuilder) -> StringBuilder:...
        @typing.overload
        def __call__(self, handler: clr.Reference[StringBuilder.AppendInterpolatedStringHandler]) -> StringBuilder:...
        @typing.overload
        def __call__(self, value: typing.Any) -> StringBuilder:...
        @typing.overload
        def __call__(self, value: str, repeatCount: int) -> StringBuilder:...
        @typing.overload
        def __call__(self, value: clr.Reference[str], valueCount: int) -> StringBuilder:...
        @typing.overload
        def __call__(self, provider: IFormatProvider, handler: clr.Reference[StringBuilder.AppendInterpolatedStringHandler]) -> StringBuilder:...
        @typing.overload
        def __call__(self, value: Array_1[str], startIndex: int, charCount: int) -> StringBuilder:...
        @typing.overload
        def __call__(self, value: str, startIndex: int, count: int) -> StringBuilder:...
        @typing.overload
        def __call__(self, value: StringBuilder, startIndex: int, count: int) -> StringBuilder:...

    # Skipped AppendFormat due to it being static, abstract and generic.

    AppendFormat : AppendFormat_MethodGroup
    class AppendFormat_MethodGroup:
        @typing.overload
        def __getitem__(self, t:typing.Type[AppendFormat_1_T1]) -> AppendFormat_1[AppendFormat_1_T1]: ...

        AppendFormat_1_T1 = typing.TypeVar('AppendFormat_1_T1')
        class AppendFormat_1(typing.Generic[AppendFormat_1_T1]):
            AppendFormat_1_TArg0 = StringBuilder.AppendFormat_MethodGroup.AppendFormat_1_T1
            def __call__(self, provider: IFormatProvider, format: CompositeFormat, arg0: AppendFormat_1_TArg0) -> StringBuilder:...

        @typing.overload
        def __getitem__(self, t:typing.Tuple[typing.Type[AppendFormat_2_T1], typing.Type[AppendFormat_2_T2]]) -> AppendFormat_2[AppendFormat_2_T1, AppendFormat_2_T2]: ...

        AppendFormat_2_T1 = typing.TypeVar('AppendFormat_2_T1')
        AppendFormat_2_T2 = typing.TypeVar('AppendFormat_2_T2')
        class AppendFormat_2(typing.Generic[AppendFormat_2_T1, AppendFormat_2_T2]):
            AppendFormat_2_TArg0 = StringBuilder.AppendFormat_MethodGroup.AppendFormat_2_T1
            AppendFormat_2_TArg1 = StringBuilder.AppendFormat_MethodGroup.AppendFormat_2_T2
            def __call__(self, provider: IFormatProvider, format: CompositeFormat, arg0: AppendFormat_2_TArg0, arg1: AppendFormat_2_TArg1) -> StringBuilder:...

        @typing.overload
        def __getitem__(self, t:typing.Tuple[typing.Type[AppendFormat_3_T1], typing.Type[AppendFormat_3_T2], typing.Type[AppendFormat_3_T3]]) -> AppendFormat_3[AppendFormat_3_T1, AppendFormat_3_T2, AppendFormat_3_T3]: ...

        AppendFormat_3_T1 = typing.TypeVar('AppendFormat_3_T1')
        AppendFormat_3_T2 = typing.TypeVar('AppendFormat_3_T2')
        AppendFormat_3_T3 = typing.TypeVar('AppendFormat_3_T3')
        class AppendFormat_3(typing.Generic[AppendFormat_3_T1, AppendFormat_3_T2, AppendFormat_3_T3]):
            AppendFormat_3_TArg0 = StringBuilder.AppendFormat_MethodGroup.AppendFormat_3_T1
            AppendFormat_3_TArg1 = StringBuilder.AppendFormat_MethodGroup.AppendFormat_3_T2
            AppendFormat_3_TArg2 = StringBuilder.AppendFormat_MethodGroup.AppendFormat_3_T3
            def __call__(self, provider: IFormatProvider, format: CompositeFormat, arg0: AppendFormat_3_TArg0, arg1: AppendFormat_3_TArg1, arg2: AppendFormat_3_TArg2) -> StringBuilder:...

        @typing.overload
        def __call__(self, format: str, args: Array_1[typing.Any]) -> StringBuilder:...
        @typing.overload
        def __call__(self, format: str, arg0: typing.Any) -> StringBuilder:...
        @typing.overload
        def __call__(self, format: str, arg0: typing.Any, arg1: typing.Any) -> StringBuilder:...
        @typing.overload
        def __call__(self, provider: IFormatProvider, format: str, args: Array_1[typing.Any]) -> StringBuilder:...
        @typing.overload
        def __call__(self, provider: IFormatProvider, format: str, arg0: typing.Any) -> StringBuilder:...
        @typing.overload
        def __call__(self, provider: IFormatProvider, format: CompositeFormat, args: Array_1[typing.Any]) -> StringBuilder:...
        @typing.overload
        def __call__(self, provider: IFormatProvider, format: CompositeFormat, args: ReadOnlySpan_1[typing.Any]) -> StringBuilder:...
        @typing.overload
        def __call__(self, format: str, arg0: typing.Any, arg1: typing.Any, arg2: typing.Any) -> StringBuilder:...
        @typing.overload
        def __call__(self, provider: IFormatProvider, format: str, arg0: typing.Any, arg1: typing.Any) -> StringBuilder:...
        @typing.overload
        def __call__(self, provider: IFormatProvider, format: str, arg0: typing.Any, arg1: typing.Any, arg2: typing.Any) -> StringBuilder:...

    # Skipped AppendJoin due to it being static, abstract and generic.

    AppendJoin : AppendJoin_MethodGroup
    class AppendJoin_MethodGroup:
        def __getitem__(self, t:typing.Type[AppendJoin_1_T1]) -> AppendJoin_1[AppendJoin_1_T1]: ...

        AppendJoin_1_T1 = typing.TypeVar('AppendJoin_1_T1')
        class AppendJoin_1(typing.Generic[AppendJoin_1_T1]):
            AppendJoin_1_T = StringBuilder.AppendJoin_MethodGroup.AppendJoin_1_T1
            def __call__(self, separator: str, values: IEnumerable_1[AppendJoin_1_T]) -> StringBuilder:...
            # Method AppendJoin(separator : Char, values : IEnumerable`1) was skipped since it collides with above method

        @typing.overload
        def __call__(self, separator: str, values: Array_1[str]) -> StringBuilder:...
        # Method AppendJoin(separator : Char, values : String[]) was skipped since it collides with above method
        @typing.overload
        def __call__(self, separator: str, values: Array_1[typing.Any]) -> StringBuilder:...
        # Method AppendJoin(separator : Char, values : Object[]) was skipped since it collides with above method

    # Skipped AppendLine due to it being static, abstract and generic.

    AppendLine : AppendLine_MethodGroup
    class AppendLine_MethodGroup:
        @typing.overload
        def __call__(self) -> StringBuilder:...
        @typing.overload
        def __call__(self, value: str) -> StringBuilder:...
        @typing.overload
        def __call__(self, handler: clr.Reference[StringBuilder.AppendInterpolatedStringHandler]) -> StringBuilder:...
        @typing.overload
        def __call__(self, provider: IFormatProvider, handler: clr.Reference[StringBuilder.AppendInterpolatedStringHandler]) -> StringBuilder:...

    # Skipped CopyTo due to it being static, abstract and generic.

    CopyTo : CopyTo_MethodGroup
    class CopyTo_MethodGroup:
        @typing.overload
        def __call__(self, sourceIndex: int, destination: Span_1[str], count: int) -> None:...
        @typing.overload
        def __call__(self, sourceIndex: int, destination: Array_1[str], destinationIndex: int, count: int) -> None:...

    # Skipped Equals due to it being static, abstract and generic.

    Equals : Equals_MethodGroup
    class Equals_MethodGroup:
        @typing.overload
        def __call__(self, span: ReadOnlySpan_1[str]) -> bool:...
        @typing.overload
        def __call__(self, sb: StringBuilder) -> bool:...

    # Skipped Insert due to it being static, abstract and generic.

    Insert : Insert_MethodGroup
    class Insert_MethodGroup:
        @typing.overload
        def __call__(self, index: int, value: float) -> StringBuilder:...
        # Method Insert(index : Int32, value : Double) was skipped since it collides with above method
        # Method Insert(index : Int32, value : SByte) was skipped since it collides with above method
        # Method Insert(index : Int32, value : Byte) was skipped since it collides with above method
        # Method Insert(index : Int32, value : Int16) was skipped since it collides with above method
        @typing.overload
        def __call__(self, index: int, value: str) -> StringBuilder:...
        # Method Insert(index : Int32, value : Int32) was skipped since it collides with above method
        # Method Insert(index : Int32, value : Int64) was skipped since it collides with above method
        # Method Insert(index : Int32, value : UInt16) was skipped since it collides with above method
        # Method Insert(index : Int32, value : UInt32) was skipped since it collides with above method
        # Method Insert(index : Int32, value : UInt64) was skipped since it collides with above method
        @typing.overload
        def __call__(self, index: int, value: Decimal) -> StringBuilder:...
        @typing.overload
        def __call__(self, index: int, value: Array_1[str]) -> StringBuilder:...
        @typing.overload
        def __call__(self, index: int, value: ReadOnlySpan_1[str]) -> StringBuilder:...
        # Method Insert(index : Int32, value : String) was skipped since it collides with above method
        # Method Insert(index : Int32, value : Boolean) was skipped since it collides with above method
        @typing.overload
        def __call__(self, index: int, value: typing.Any) -> StringBuilder:...
        @typing.overload
        def __call__(self, index: int, value: str, count: int) -> StringBuilder:...
        @typing.overload
        def __call__(self, index: int, value: Array_1[str], startIndex: int, charCount: int) -> StringBuilder:...

    # Skipped Replace due to it being static, abstract and generic.

    Replace : Replace_MethodGroup
    class Replace_MethodGroup:
        @typing.overload
        def __call__(self, oldChar: str, newChar: str) -> StringBuilder:...
        # Method Replace(oldValue : String, newValue : String) was skipped since it collides with above method
        @typing.overload
        def __call__(self, oldChar: str, newChar: str, startIndex: int, count: int) -> StringBuilder:...
        # Method Replace(oldValue : String, newValue : String, startIndex : Int32, count : Int32) was skipped since it collides with above method

    # Skipped ToString due to it being static, abstract and generic.

    ToString : ToString_MethodGroup
    class ToString_MethodGroup:
        @typing.overload
        def __call__(self) -> str:...
        @typing.overload
        def __call__(self, startIndex: int, length: int) -> str:...


    class AppendInterpolatedStringHandler:
        @typing.overload
        def __init__(self, literalLength: int, formattedCount: int, stringBuilder: StringBuilder) -> None: ...
        @typing.overload
        def __init__(self, literalLength: int, formattedCount: int, stringBuilder: StringBuilder, provider: IFormatProvider) -> None: ...
        def AppendLiteral(self, value: str) -> None: ...
        # Skipped AppendFormatted due to it being static, abstract and generic.

        AppendFormatted : AppendFormatted_MethodGroup
        class AppendFormatted_MethodGroup:
            def __getitem__(self, t:typing.Type[AppendFormatted_1_T1]) -> AppendFormatted_1[AppendFormatted_1_T1]: ...

            AppendFormatted_1_T1 = typing.TypeVar('AppendFormatted_1_T1')
            class AppendFormatted_1(typing.Generic[AppendFormatted_1_T1]):
                AppendFormatted_1_T = StringBuilder.AppendInterpolatedStringHandler.AppendFormatted_MethodGroup.AppendFormatted_1_T1
                @typing.overload
                def __call__(self, value: AppendFormatted_1_T) -> None:...
                @typing.overload
                def __call__(self, value: AppendFormatted_1_T, alignment: int) -> None:...
                @typing.overload
                def __call__(self, value: AppendFormatted_1_T, format: str) -> None:...
                @typing.overload
                def __call__(self, value: AppendFormatted_1_T, alignment: int, format: str) -> None:...

            @typing.overload
            def __call__(self, value: ReadOnlySpan_1[str]) -> None:...
            @typing.overload
            def __call__(self, value: str) -> None:...
            @typing.overload
            def __call__(self, value: ReadOnlySpan_1[str], alignment: int = ..., format: str = ...) -> None:...
            @typing.overload
            def __call__(self, value: str, alignment: int = ..., format: str = ...) -> None:...
            @typing.overload
            def __call__(self, value: typing.Any, alignment: int = ..., format: str = ...) -> None:...



    class ChunkEnumerator:
        @property
        def Current(self) -> ReadOnlyMemory_1[str]: ...
        def GetEnumerator(self) -> StringBuilder.ChunkEnumerator: ...
        def MoveNext(self) -> bool: ...



class StringRuneEnumerator(IEnumerator_1[Rune], IEnumerable_1[Rune]):
    @property
    def Current(self) -> Rune: ...
    def GetEnumerator(self) -> StringRuneEnumerator: ...
    def MoveNext(self) -> bool: ...


class UnicodeEncoding(Encoding):
    @typing.overload
    def __init__(self) -> None: ...
    @typing.overload
    def __init__(self, bigEndian: bool, byteOrderMark: bool) -> None: ...
    @typing.overload
    def __init__(self, bigEndian: bool, byteOrderMark: bool, throwOnInvalidBytes: bool) -> None: ...
    CharSize : int
    @property
    def BodyName(self) -> str: ...
    @property
    def CodePage(self) -> int: ...
    @property
    def DecoderFallback(self) -> DecoderFallback: ...
    @DecoderFallback.setter
    def DecoderFallback(self, value: DecoderFallback) -> DecoderFallback: ...
    @property
    def EncoderFallback(self) -> EncoderFallback: ...
    @EncoderFallback.setter
    def EncoderFallback(self, value: EncoderFallback) -> EncoderFallback: ...
    @property
    def EncodingName(self) -> str: ...
    @property
    def HeaderName(self) -> str: ...
    @property
    def IsBrowserDisplay(self) -> bool: ...
    @property
    def IsBrowserSave(self) -> bool: ...
    @property
    def IsMailNewsDisplay(self) -> bool: ...
    @property
    def IsMailNewsSave(self) -> bool: ...
    @property
    def IsReadOnly(self) -> bool: ...
    @IsReadOnly.setter
    def IsReadOnly(self, value: bool) -> bool: ...
    @property
    def IsSingleByte(self) -> bool: ...
    @property
    def Preamble(self) -> ReadOnlySpan_1[int]: ...
    @property
    def WebName(self) -> str: ...
    @property
    def WindowsCodePage(self) -> int: ...
    def Equals(self, value: typing.Any) -> bool: ...
    def GetDecoder(self) -> Decoder: ...
    def GetEncoder(self) -> Encoder: ...
    def GetHashCode(self) -> int: ...
    def GetMaxByteCount(self, charCount: int) -> int: ...
    def GetMaxCharCount(self, byteCount: int) -> int: ...
    def GetPreamble(self) -> Array_1[int]: ...
    def GetString(self, bytes: Array_1[int], index: int, count: int) -> str: ...
    # Skipped GetByteCount due to it being static, abstract and generic.

    GetByteCount : GetByteCount_MethodGroup
    class GetByteCount_MethodGroup:
        @typing.overload
        def __call__(self, s: str) -> int:...
        @typing.overload
        def __call__(self, chars: clr.Reference[str], count: int) -> int:...
        @typing.overload
        def __call__(self, chars: Array_1[str], index: int, count: int) -> int:...

    # Skipped GetBytes due to it being static, abstract and generic.

    GetBytes : GetBytes_MethodGroup
    class GetBytes_MethodGroup:
        @typing.overload
        def __call__(self, chars: clr.Reference[str], charCount: int, bytes: clr.Reference[int], byteCount: int) -> int:...
        @typing.overload
        def __call__(self, chars: Array_1[str], charIndex: int, charCount: int, bytes: Array_1[int], byteIndex: int) -> int:...
        @typing.overload
        def __call__(self, s: str, charIndex: int, charCount: int, bytes: Array_1[int], byteIndex: int) -> int:...

    # Skipped GetCharCount due to it being static, abstract and generic.

    GetCharCount : GetCharCount_MethodGroup
    class GetCharCount_MethodGroup:
        @typing.overload
        def __call__(self, bytes: clr.Reference[int], count: int) -> int:...
        @typing.overload
        def __call__(self, bytes: Array_1[int], index: int, count: int) -> int:...

    # Skipped GetChars due to it being static, abstract and generic.

    GetChars : GetChars_MethodGroup
    class GetChars_MethodGroup:
        @typing.overload
        def __call__(self, bytes: clr.Reference[int], byteCount: int, chars: clr.Reference[str], charCount: int) -> int:...
        @typing.overload
        def __call__(self, bytes: Array_1[int], byteIndex: int, byteCount: int, chars: Array_1[str], charIndex: int) -> int:...



class UTF32Encoding(Encoding):
    @typing.overload
    def __init__(self) -> None: ...
    @typing.overload
    def __init__(self, bigEndian: bool, byteOrderMark: bool) -> None: ...
    @typing.overload
    def __init__(self, bigEndian: bool, byteOrderMark: bool, throwOnInvalidCharacters: bool) -> None: ...
    @property
    def BodyName(self) -> str: ...
    @property
    def CodePage(self) -> int: ...
    @property
    def DecoderFallback(self) -> DecoderFallback: ...
    @DecoderFallback.setter
    def DecoderFallback(self, value: DecoderFallback) -> DecoderFallback: ...
    @property
    def EncoderFallback(self) -> EncoderFallback: ...
    @EncoderFallback.setter
    def EncoderFallback(self, value: EncoderFallback) -> EncoderFallback: ...
    @property
    def EncodingName(self) -> str: ...
    @property
    def HeaderName(self) -> str: ...
    @property
    def IsBrowserDisplay(self) -> bool: ...
    @property
    def IsBrowserSave(self) -> bool: ...
    @property
    def IsMailNewsDisplay(self) -> bool: ...
    @property
    def IsMailNewsSave(self) -> bool: ...
    @property
    def IsReadOnly(self) -> bool: ...
    @IsReadOnly.setter
    def IsReadOnly(self, value: bool) -> bool: ...
    @property
    def IsSingleByte(self) -> bool: ...
    @property
    def Preamble(self) -> ReadOnlySpan_1[int]: ...
    @property
    def WebName(self) -> str: ...
    @property
    def WindowsCodePage(self) -> int: ...
    def Equals(self, value: typing.Any) -> bool: ...
    def GetDecoder(self) -> Decoder: ...
    def GetEncoder(self) -> Encoder: ...
    def GetHashCode(self) -> int: ...
    def GetMaxByteCount(self, charCount: int) -> int: ...
    def GetMaxCharCount(self, byteCount: int) -> int: ...
    def GetPreamble(self) -> Array_1[int]: ...
    def GetString(self, bytes: Array_1[int], index: int, count: int) -> str: ...
    # Skipped GetByteCount due to it being static, abstract and generic.

    GetByteCount : GetByteCount_MethodGroup
    class GetByteCount_MethodGroup:
        @typing.overload
        def __call__(self, s: str) -> int:...
        @typing.overload
        def __call__(self, chars: clr.Reference[str], count: int) -> int:...
        @typing.overload
        def __call__(self, chars: Array_1[str], index: int, count: int) -> int:...

    # Skipped GetBytes due to it being static, abstract and generic.

    GetBytes : GetBytes_MethodGroup
    class GetBytes_MethodGroup:
        @typing.overload
        def __call__(self, chars: clr.Reference[str], charCount: int, bytes: clr.Reference[int], byteCount: int) -> int:...
        @typing.overload
        def __call__(self, chars: Array_1[str], charIndex: int, charCount: int, bytes: Array_1[int], byteIndex: int) -> int:...
        @typing.overload
        def __call__(self, s: str, charIndex: int, charCount: int, bytes: Array_1[int], byteIndex: int) -> int:...

    # Skipped GetCharCount due to it being static, abstract and generic.

    GetCharCount : GetCharCount_MethodGroup
    class GetCharCount_MethodGroup:
        @typing.overload
        def __call__(self, bytes: clr.Reference[int], count: int) -> int:...
        @typing.overload
        def __call__(self, bytes: Array_1[int], index: int, count: int) -> int:...

    # Skipped GetChars due to it being static, abstract and generic.

    GetChars : GetChars_MethodGroup
    class GetChars_MethodGroup:
        @typing.overload
        def __call__(self, bytes: clr.Reference[int], byteCount: int, chars: clr.Reference[str], charCount: int) -> int:...
        @typing.overload
        def __call__(self, bytes: Array_1[int], byteIndex: int, byteCount: int, chars: Array_1[str], charIndex: int) -> int:...



class UTF7Encoding(Encoding):
    @typing.overload
    def __init__(self) -> None: ...
    @typing.overload
    def __init__(self, allowOptionals: bool) -> None: ...
    @property
    def BodyName(self) -> str: ...
    @property
    def CodePage(self) -> int: ...
    @property
    def DecoderFallback(self) -> DecoderFallback: ...
    @DecoderFallback.setter
    def DecoderFallback(self, value: DecoderFallback) -> DecoderFallback: ...
    @property
    def EncoderFallback(self) -> EncoderFallback: ...
    @EncoderFallback.setter
    def EncoderFallback(self, value: EncoderFallback) -> EncoderFallback: ...
    @property
    def EncodingName(self) -> str: ...
    @property
    def HeaderName(self) -> str: ...
    @property
    def IsBrowserDisplay(self) -> bool: ...
    @property
    def IsBrowserSave(self) -> bool: ...
    @property
    def IsMailNewsDisplay(self) -> bool: ...
    @property
    def IsMailNewsSave(self) -> bool: ...
    @property
    def IsReadOnly(self) -> bool: ...
    @IsReadOnly.setter
    def IsReadOnly(self, value: bool) -> bool: ...
    @property
    def IsSingleByte(self) -> bool: ...
    @property
    def Preamble(self) -> ReadOnlySpan_1[int]: ...
    @property
    def WebName(self) -> str: ...
    @property
    def WindowsCodePage(self) -> int: ...
    def Equals(self, value: typing.Any) -> bool: ...
    def GetDecoder(self) -> Decoder: ...
    def GetEncoder(self) -> Encoder: ...
    def GetHashCode(self) -> int: ...
    def GetMaxByteCount(self, charCount: int) -> int: ...
    def GetMaxCharCount(self, byteCount: int) -> int: ...
    def GetString(self, bytes: Array_1[int], index: int, count: int) -> str: ...
    # Skipped GetByteCount due to it being static, abstract and generic.

    GetByteCount : GetByteCount_MethodGroup
    class GetByteCount_MethodGroup:
        @typing.overload
        def __call__(self, s: str) -> int:...
        @typing.overload
        def __call__(self, chars: clr.Reference[str], count: int) -> int:...
        @typing.overload
        def __call__(self, chars: Array_1[str], index: int, count: int) -> int:...

    # Skipped GetBytes due to it being static, abstract and generic.

    GetBytes : GetBytes_MethodGroup
    class GetBytes_MethodGroup:
        @typing.overload
        def __call__(self, chars: clr.Reference[str], charCount: int, bytes: clr.Reference[int], byteCount: int) -> int:...
        @typing.overload
        def __call__(self, chars: Array_1[str], charIndex: int, charCount: int, bytes: Array_1[int], byteIndex: int) -> int:...
        @typing.overload
        def __call__(self, s: str, charIndex: int, charCount: int, bytes: Array_1[int], byteIndex: int) -> int:...

    # Skipped GetCharCount due to it being static, abstract and generic.

    GetCharCount : GetCharCount_MethodGroup
    class GetCharCount_MethodGroup:
        @typing.overload
        def __call__(self, bytes: clr.Reference[int], count: int) -> int:...
        @typing.overload
        def __call__(self, bytes: Array_1[int], index: int, count: int) -> int:...

    # Skipped GetChars due to it being static, abstract and generic.

    GetChars : GetChars_MethodGroup
    class GetChars_MethodGroup:
        @typing.overload
        def __call__(self, bytes: clr.Reference[int], byteCount: int, chars: clr.Reference[str], charCount: int) -> int:...
        @typing.overload
        def __call__(self, bytes: Array_1[int], byteIndex: int, byteCount: int, chars: Array_1[str], charIndex: int) -> int:...



class UTF8Encoding(Encoding):
    @typing.overload
    def __init__(self) -> None: ...
    @typing.overload
    def __init__(self, encoderShouldEmitUTF8Identifier: bool) -> None: ...
    @typing.overload
    def __init__(self, encoderShouldEmitUTF8Identifier: bool, throwOnInvalidBytes: bool) -> None: ...
    @property
    def BodyName(self) -> str: ...
    @property
    def CodePage(self) -> int: ...
    @property
    def DecoderFallback(self) -> DecoderFallback: ...
    @DecoderFallback.setter
    def DecoderFallback(self, value: DecoderFallback) -> DecoderFallback: ...
    @property
    def EncoderFallback(self) -> EncoderFallback: ...
    @EncoderFallback.setter
    def EncoderFallback(self, value: EncoderFallback) -> EncoderFallback: ...
    @property
    def EncodingName(self) -> str: ...
    @property
    def HeaderName(self) -> str: ...
    @property
    def IsBrowserDisplay(self) -> bool: ...
    @property
    def IsBrowserSave(self) -> bool: ...
    @property
    def IsMailNewsDisplay(self) -> bool: ...
    @property
    def IsMailNewsSave(self) -> bool: ...
    @property
    def IsReadOnly(self) -> bool: ...
    @IsReadOnly.setter
    def IsReadOnly(self, value: bool) -> bool: ...
    @property
    def IsSingleByte(self) -> bool: ...
    @property
    def Preamble(self) -> ReadOnlySpan_1[int]: ...
    @property
    def WebName(self) -> str: ...
    @property
    def WindowsCodePage(self) -> int: ...
    def Equals(self, value: typing.Any) -> bool: ...
    def GetDecoder(self) -> Decoder: ...
    def GetEncoder(self) -> Encoder: ...
    def GetHashCode(self) -> int: ...
    def GetMaxByteCount(self, charCount: int) -> int: ...
    def GetMaxCharCount(self, byteCount: int) -> int: ...
    def GetPreamble(self) -> Array_1[int]: ...
    def GetString(self, bytes: Array_1[int], index: int, count: int) -> str: ...
    def TryGetBytes(self, chars: ReadOnlySpan_1[str], bytes: Span_1[int], bytesWritten: clr.Reference[int]) -> bool: ...
    def TryGetChars(self, bytes: ReadOnlySpan_1[int], chars: Span_1[str], charsWritten: clr.Reference[int]) -> bool: ...
    # Skipped GetByteCount due to it being static, abstract and generic.

    GetByteCount : GetByteCount_MethodGroup
    class GetByteCount_MethodGroup:
        @typing.overload
        def __call__(self, chars: ReadOnlySpan_1[str]) -> int:...
        @typing.overload
        def __call__(self, chars: str) -> int:...
        @typing.overload
        def __call__(self, chars: clr.Reference[str], count: int) -> int:...
        @typing.overload
        def __call__(self, chars: Array_1[str], index: int, count: int) -> int:...

    # Skipped GetBytes due to it being static, abstract and generic.

    GetBytes : GetBytes_MethodGroup
    class GetBytes_MethodGroup:
        @typing.overload
        def __call__(self, chars: ReadOnlySpan_1[str], bytes: Span_1[int]) -> int:...
        @typing.overload
        def __call__(self, chars: clr.Reference[str], charCount: int, bytes: clr.Reference[int], byteCount: int) -> int:...
        @typing.overload
        def __call__(self, chars: Array_1[str], charIndex: int, charCount: int, bytes: Array_1[int], byteIndex: int) -> int:...
        @typing.overload
        def __call__(self, s: str, charIndex: int, charCount: int, bytes: Array_1[int], byteIndex: int) -> int:...

    # Skipped GetCharCount due to it being static, abstract and generic.

    GetCharCount : GetCharCount_MethodGroup
    class GetCharCount_MethodGroup:
        @typing.overload
        def __call__(self, bytes: ReadOnlySpan_1[int]) -> int:...
        @typing.overload
        def __call__(self, bytes: clr.Reference[int], count: int) -> int:...
        @typing.overload
        def __call__(self, bytes: Array_1[int], index: int, count: int) -> int:...

    # Skipped GetChars due to it being static, abstract and generic.

    GetChars : GetChars_MethodGroup
    class GetChars_MethodGroup:
        @typing.overload
        def __call__(self, bytes: ReadOnlySpan_1[int], chars: Span_1[str]) -> int:...
        @typing.overload
        def __call__(self, bytes: clr.Reference[int], byteCount: int, chars: clr.Reference[str], charCount: int) -> int:...
        @typing.overload
        def __call__(self, bytes: Array_1[int], byteIndex: int, byteCount: int, chars: Array_1[str], charIndex: int) -> int:...





import typing, abc

class IRandomizer(typing.Protocol):
    @abc.abstractmethod
    def Next(self) -> float: ...


class NormalDistributedRandomizer(IRandomizer):
    def __init__(self) -> None: ...
    def Next(self) -> float: ...




import typing, abc
from System.IO import FileInfo
from SIMULTAN.Serializer.DXF import DXFParserInfo
from SIMULTAN.Projects import ProjectData
from System.Xml import XmlElement, XmlNamespaceManager, XmlWriter
from System import Action_1

class ExcelDxfIO(abc.ABC):
    @staticmethod
    def Read(file: FileInfo, parserInfo: DXFParserInfo) -> None: ...
    @staticmethod
    def ReadLibrary(file: FileInfo, parserInfo: DXFParserInfo) -> None: ...
    @staticmethod
    def Write(file: FileInfo, projectData: ProjectData) -> None: ...


class XMLIOExtensions(abc.ABC):
    @staticmethod
    def LoadInnerText(node: XmlElement, xpath: str, converter: Action_1[str], nsmgr: XmlNamespaceManager = ...) -> bool: ...
    # Skipped WriteKeyValue due to it being static, abstract and generic.

    WriteKeyValue : WriteKeyValue_MethodGroup
    class WriteKeyValue_MethodGroup:
        def __getitem__(self, t:typing.Type[WriteKeyValue_1_T1]) -> WriteKeyValue_1[WriteKeyValue_1_T1]: ...

        WriteKeyValue_1_T1 = typing.TypeVar('WriteKeyValue_1_T1')
        class WriteKeyValue_1(typing.Generic[WriteKeyValue_1_T1]):
            WriteKeyValue_1_T = XMLIOExtensions.WriteKeyValue_MethodGroup.WriteKeyValue_1_T1
            def __call__(self, sw: XmlWriter, key: str, value: WriteKeyValue_1_T) -> None:...






import typing, clr, abc
from System import Guid, Array_1

class BIND_OPTS:
    cbStruct : int
    dwTickCountDeadline : int
    grfFlags : int
    grfMode : int


class BINDPTR:
    lpfuncdesc : int
    lptcomp : int
    lpvardesc : int


class CALLCONV(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    CC_CDECL : CALLCONV # 1
    CC_MSCPASCAL : CALLCONV # 2
    CC_PASCAL : CALLCONV # 2
    CC_MACPASCAL : CALLCONV # 3
    CC_STDCALL : CALLCONV # 4
    CC_RESERVED : CALLCONV # 5
    CC_SYSCALL : CALLCONV # 6
    CC_MPWCDECL : CALLCONV # 7
    CC_MPWPASCAL : CALLCONV # 8
    CC_MAX : CALLCONV # 9


class CONNECTDATA:
    dwCookie : int
    pUnk : typing.Any


class DESCKIND(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    DESCKIND_NONE : DESCKIND # 0
    DESCKIND_FUNCDESC : DESCKIND # 1
    DESCKIND_VARDESC : DESCKIND # 2
    DESCKIND_TYPECOMP : DESCKIND # 3
    DESCKIND_IMPLICITAPPOBJ : DESCKIND # 4
    DESCKIND_MAX : DESCKIND # 5


class DISPPARAMS:
    cArgs : int
    cNamedArgs : int
    rgdispidNamedArgs : int
    rgvarg : int


class ELEMDESC:
    desc : ELEMDESC.DESCUNION
    tdesc : TYPEDESC

    class DESCUNION:
        idldesc : IDLDESC
        paramdesc : PARAMDESC



class EXCEPINFO:
    bstrDescription : str
    bstrHelpFile : str
    bstrSource : str
    dwHelpContext : int
    pfnDeferredFillIn : int
    pvReserved : int
    scode : int
    wCode : int
    wReserved : int


class FILETIME:
    dwHighDateTime : int
    dwLowDateTime : int


class FUNCDESC:
    callconv : CALLCONV
    cParams : int
    cParamsOpt : int
    cScodes : int
    elemdescFunc : ELEMDESC
    funckind : FUNCKIND
    invkind : INVOKEKIND
    lprgelemdescParam : int
    lprgscode : int
    memid : int
    oVft : int
    wFuncFlags : int


class FUNCFLAGS(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    FUNCFLAG_FRESTRICTED : FUNCFLAGS # 1
    FUNCFLAG_FSOURCE : FUNCFLAGS # 2
    FUNCFLAG_FBINDABLE : FUNCFLAGS # 4
    FUNCFLAG_FREQUESTEDIT : FUNCFLAGS # 8
    FUNCFLAG_FDISPLAYBIND : FUNCFLAGS # 16
    FUNCFLAG_FDEFAULTBIND : FUNCFLAGS # 32
    FUNCFLAG_FHIDDEN : FUNCFLAGS # 64
    FUNCFLAG_FUSESGETLASTERROR : FUNCFLAGS # 128
    FUNCFLAG_FDEFAULTCOLLELEM : FUNCFLAGS # 256
    FUNCFLAG_FUIDEFAULT : FUNCFLAGS # 512
    FUNCFLAG_FNONBROWSABLE : FUNCFLAGS # 1024
    FUNCFLAG_FREPLACEABLE : FUNCFLAGS # 2048
    FUNCFLAG_FIMMEDIATEBIND : FUNCFLAGS # 4096


class FUNCKIND(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    FUNC_VIRTUAL : FUNCKIND # 0
    FUNC_PUREVIRTUAL : FUNCKIND # 1
    FUNC_NONVIRTUAL : FUNCKIND # 2
    FUNC_STATIC : FUNCKIND # 3
    FUNC_DISPATCH : FUNCKIND # 4


class IBindCtx(typing.Protocol):
    @abc.abstractmethod
    def EnumObjectParam(self, ppenum: clr.Reference[IEnumString]) -> None: ...
    @abc.abstractmethod
    def GetBindOptions(self, pbindopts: clr.Reference[BIND_OPTS]) -> None: ...
    @abc.abstractmethod
    def GetObjectParam(self, pszKey: str, ppunk: clr.Reference[typing.Any]) -> None: ...
    @abc.abstractmethod
    def GetRunningObjectTable(self, pprot: clr.Reference[IRunningObjectTable]) -> None: ...
    @abc.abstractmethod
    def RegisterObjectBound(self, punk: typing.Any) -> None: ...
    @abc.abstractmethod
    def RegisterObjectParam(self, pszKey: str, punk: typing.Any) -> None: ...
    @abc.abstractmethod
    def ReleaseBoundObjects(self) -> None: ...
    @abc.abstractmethod
    def RevokeObjectBound(self, punk: typing.Any) -> None: ...
    @abc.abstractmethod
    def RevokeObjectParam(self, pszKey: str) -> int: ...
    @abc.abstractmethod
    def SetBindOptions(self, pbindopts: clr.Reference[BIND_OPTS]) -> None: ...


class IConnectionPoint(typing.Protocol):
    @abc.abstractmethod
    def Advise(self, pUnkSink: typing.Any, pdwCookie: clr.Reference[int]) -> None: ...
    @abc.abstractmethod
    def EnumConnections(self, ppEnum: clr.Reference[IEnumConnections]) -> None: ...
    @abc.abstractmethod
    def GetConnectionInterface(self, pIID: clr.Reference[Guid]) -> None: ...
    @abc.abstractmethod
    def GetConnectionPointContainer(self, ppCPC: clr.Reference[IConnectionPointContainer]) -> None: ...
    @abc.abstractmethod
    def Unadvise(self, dwCookie: int) -> None: ...


class IConnectionPointContainer(typing.Protocol):
    @abc.abstractmethod
    def EnumConnectionPoints(self, ppEnum: clr.Reference[IEnumConnectionPoints]) -> None: ...
    @abc.abstractmethod
    def FindConnectionPoint(self, riid: clr.Reference[Guid], ppCP: clr.Reference[IConnectionPoint]) -> None: ...


class IDLDESC:
    dwReserved : int
    wIDLFlags : IDLFLAG


class IDLFLAG(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    IDLFLAG_NONE : IDLFLAG # 0
    IDLFLAG_FIN : IDLFLAG # 1
    IDLFLAG_FOUT : IDLFLAG # 2
    IDLFLAG_FLCID : IDLFLAG # 4
    IDLFLAG_FRETVAL : IDLFLAG # 8


class IEnumConnectionPoints(typing.Protocol):
    @abc.abstractmethod
    def Clone(self, ppenum: clr.Reference[IEnumConnectionPoints]) -> None: ...
    @abc.abstractmethod
    def Next(self, celt: int, rgelt: Array_1[IConnectionPoint], pceltFetched: int) -> int: ...
    @abc.abstractmethod
    def Reset(self) -> None: ...
    @abc.abstractmethod
    def Skip(self, celt: int) -> int: ...


class IEnumConnections(typing.Protocol):
    @abc.abstractmethod
    def Clone(self, ppenum: clr.Reference[IEnumConnections]) -> None: ...
    @abc.abstractmethod
    def Next(self, celt: int, rgelt: Array_1[CONNECTDATA], pceltFetched: int) -> int: ...
    @abc.abstractmethod
    def Reset(self) -> None: ...
    @abc.abstractmethod
    def Skip(self, celt: int) -> int: ...


class IEnumMoniker(typing.Protocol):
    @abc.abstractmethod
    def Clone(self, ppenum: clr.Reference[IEnumMoniker]) -> None: ...
    @abc.abstractmethod
    def Next(self, celt: int, rgelt: Array_1[IMoniker], pceltFetched: int) -> int: ...
    @abc.abstractmethod
    def Reset(self) -> None: ...
    @abc.abstractmethod
    def Skip(self, celt: int) -> int: ...


class IEnumString(typing.Protocol):
    @abc.abstractmethod
    def Clone(self, ppenum: clr.Reference[IEnumString]) -> None: ...
    @abc.abstractmethod
    def Next(self, celt: int, rgelt: Array_1[str], pceltFetched: int) -> int: ...
    @abc.abstractmethod
    def Reset(self) -> None: ...
    @abc.abstractmethod
    def Skip(self, celt: int) -> int: ...


class IEnumVARIANT(typing.Protocol):
    @abc.abstractmethod
    def Clone(self) -> IEnumVARIANT: ...
    @abc.abstractmethod
    def Next(self, celt: int, rgVar: Array_1[typing.Any], pceltFetched: int) -> int: ...
    @abc.abstractmethod
    def Reset(self) -> int: ...
    @abc.abstractmethod
    def Skip(self, celt: int) -> int: ...


class IMoniker(typing.Protocol):
    @abc.abstractmethod
    def BindToObject(self, pbc: IBindCtx, pmkToLeft: IMoniker, riidResult: clr.Reference[Guid], ppvResult: clr.Reference[typing.Any]) -> None: ...
    @abc.abstractmethod
    def BindToStorage(self, pbc: IBindCtx, pmkToLeft: IMoniker, riid: clr.Reference[Guid], ppvObj: clr.Reference[typing.Any]) -> None: ...
    @abc.abstractmethod
    def CommonPrefixWith(self, pmkOther: IMoniker, ppmkPrefix: clr.Reference[IMoniker]) -> None: ...
    @abc.abstractmethod
    def ComposeWith(self, pmkRight: IMoniker, fOnlyIfNotGeneric: bool, ppmkComposite: clr.Reference[IMoniker]) -> None: ...
    @abc.abstractmethod
    def Enum(self, fForward: bool, ppenumMoniker: clr.Reference[IEnumMoniker]) -> None: ...
    @abc.abstractmethod
    def GetClassID(self, pClassID: clr.Reference[Guid]) -> None: ...
    @abc.abstractmethod
    def GetDisplayName(self, pbc: IBindCtx, pmkToLeft: IMoniker, ppszDisplayName: clr.Reference[str]) -> None: ...
    @abc.abstractmethod
    def GetSizeMax(self, pcbSize: clr.Reference[int]) -> None: ...
    @abc.abstractmethod
    def GetTimeOfLastChange(self, pbc: IBindCtx, pmkToLeft: IMoniker, pFileTime: clr.Reference[FILETIME]) -> None: ...
    @abc.abstractmethod
    def Hash(self, pdwHash: clr.Reference[int]) -> None: ...
    @abc.abstractmethod
    def Inverse(self, ppmk: clr.Reference[IMoniker]) -> None: ...
    @abc.abstractmethod
    def IsDirty(self) -> int: ...
    @abc.abstractmethod
    def IsEqual(self, pmkOtherMoniker: IMoniker) -> int: ...
    @abc.abstractmethod
    def IsRunning(self, pbc: IBindCtx, pmkToLeft: IMoniker, pmkNewlyRunning: IMoniker) -> int: ...
    @abc.abstractmethod
    def IsSystemMoniker(self, pdwMksys: clr.Reference[int]) -> int: ...
    @abc.abstractmethod
    def Load(self, pStm: IStream) -> None: ...
    @abc.abstractmethod
    def ParseDisplayName(self, pbc: IBindCtx, pmkToLeft: IMoniker, pszDisplayName: str, pchEaten: clr.Reference[int], ppmkOut: clr.Reference[IMoniker]) -> None: ...
    @abc.abstractmethod
    def Reduce(self, pbc: IBindCtx, dwReduceHowFar: int, ppmkToLeft: clr.Reference[IMoniker], ppmkReduced: clr.Reference[IMoniker]) -> None: ...
    @abc.abstractmethod
    def RelativePathTo(self, pmkOther: IMoniker, ppmkRelPath: clr.Reference[IMoniker]) -> None: ...
    @abc.abstractmethod
    def Save(self, pStm: IStream, fClearDirty: bool) -> None: ...


class IMPLTYPEFLAGS(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    IMPLTYPEFLAG_FDEFAULT : IMPLTYPEFLAGS # 1
    IMPLTYPEFLAG_FSOURCE : IMPLTYPEFLAGS # 2
    IMPLTYPEFLAG_FRESTRICTED : IMPLTYPEFLAGS # 4
    IMPLTYPEFLAG_FDEFAULTVTABLE : IMPLTYPEFLAGS # 8


class INVOKEKIND(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    INVOKE_FUNC : INVOKEKIND # 1
    INVOKE_PROPERTYGET : INVOKEKIND # 2
    INVOKE_PROPERTYPUT : INVOKEKIND # 4
    INVOKE_PROPERTYPUTREF : INVOKEKIND # 8


class IPersistFile(typing.Protocol):
    @abc.abstractmethod
    def GetClassID(self, pClassID: clr.Reference[Guid]) -> None: ...
    @abc.abstractmethod
    def GetCurFile(self, ppszFileName: clr.Reference[str]) -> None: ...
    @abc.abstractmethod
    def IsDirty(self) -> int: ...
    @abc.abstractmethod
    def Load(self, pszFileName: str, dwMode: int) -> None: ...
    @abc.abstractmethod
    def Save(self, pszFileName: str, fRemember: bool) -> None: ...
    @abc.abstractmethod
    def SaveCompleted(self, pszFileName: str) -> None: ...


class IRunningObjectTable(typing.Protocol):
    @abc.abstractmethod
    def EnumRunning(self, ppenumMoniker: clr.Reference[IEnumMoniker]) -> None: ...
    @abc.abstractmethod
    def GetObject(self, pmkObjectName: IMoniker, ppunkObject: clr.Reference[typing.Any]) -> int: ...
    @abc.abstractmethod
    def GetTimeOfLastChange(self, pmkObjectName: IMoniker, pfiletime: clr.Reference[FILETIME]) -> int: ...
    @abc.abstractmethod
    def IsRunning(self, pmkObjectName: IMoniker) -> int: ...
    @abc.abstractmethod
    def NoteChangeTime(self, dwRegister: int, pfiletime: clr.Reference[FILETIME]) -> None: ...
    @abc.abstractmethod
    def Register(self, grfFlags: int, punkObject: typing.Any, pmkObjectName: IMoniker) -> int: ...
    @abc.abstractmethod
    def Revoke(self, dwRegister: int) -> None: ...


class IStream(typing.Protocol):
    @abc.abstractmethod
    def Clone(self, ppstm: clr.Reference[IStream]) -> None: ...
    @abc.abstractmethod
    def Commit(self, grfCommitFlags: int) -> None: ...
    @abc.abstractmethod
    def CopyTo(self, pstm: IStream, cb: int, pcbRead: int, pcbWritten: int) -> None: ...
    @abc.abstractmethod
    def LockRegion(self, libOffset: int, cb: int, dwLockType: int) -> None: ...
    @abc.abstractmethod
    def Read(self, pv: Array_1[int], cb: int, pcbRead: int) -> None: ...
    @abc.abstractmethod
    def Revert(self) -> None: ...
    @abc.abstractmethod
    def Seek(self, dlibMove: int, dwOrigin: int, plibNewPosition: int) -> None: ...
    @abc.abstractmethod
    def SetSize(self, libNewSize: int) -> None: ...
    @abc.abstractmethod
    def Stat(self, pstatstg: clr.Reference[STATSTG], grfStatFlag: int) -> None: ...
    @abc.abstractmethod
    def UnlockRegion(self, libOffset: int, cb: int, dwLockType: int) -> None: ...
    @abc.abstractmethod
    def Write(self, pv: Array_1[int], cb: int, pcbWritten: int) -> None: ...


class ITypeComp(typing.Protocol):
    @abc.abstractmethod
    def Bind(self, szName: str, lHashVal: int, wFlags: int, ppTInfo: clr.Reference[ITypeInfo], pDescKind: clr.Reference[DESCKIND], pBindPtr: clr.Reference[BINDPTR]) -> None: ...
    @abc.abstractmethod
    def BindType(self, szName: str, lHashVal: int, ppTInfo: clr.Reference[ITypeInfo], ppTComp: clr.Reference[ITypeComp]) -> None: ...


class ITypeInfo(typing.Protocol):
    @abc.abstractmethod
    def AddressOfMember(self, memid: int, invKind: INVOKEKIND, ppv: clr.Reference[int]) -> None: ...
    @abc.abstractmethod
    def CreateInstance(self, pUnkOuter: typing.Any, riid: clr.Reference[Guid], ppvObj: clr.Reference[typing.Any]) -> None: ...
    @abc.abstractmethod
    def GetContainingTypeLib(self, ppTLB: clr.Reference[ITypeLib], pIndex: clr.Reference[int]) -> None: ...
    @abc.abstractmethod
    def GetDllEntry(self, memid: int, invKind: INVOKEKIND, pBstrDllName: int, pBstrName: int, pwOrdinal: int) -> None: ...
    @abc.abstractmethod
    def GetDocumentation(self, index: int, strName: clr.Reference[str], strDocString: clr.Reference[str], dwHelpContext: clr.Reference[int], strHelpFile: clr.Reference[str]) -> None: ...
    @abc.abstractmethod
    def GetFuncDesc(self, index: int, ppFuncDesc: clr.Reference[int]) -> None: ...
    @abc.abstractmethod
    def GetIDsOfNames(self, rgszNames: Array_1[str], cNames: int, pMemId: Array_1[int]) -> None: ...
    @abc.abstractmethod
    def GetImplTypeFlags(self, index: int, pImplTypeFlags: clr.Reference[IMPLTYPEFLAGS]) -> None: ...
    @abc.abstractmethod
    def GetMops(self, memid: int, pBstrMops: clr.Reference[str]) -> None: ...
    @abc.abstractmethod
    def GetNames(self, memid: int, rgBstrNames: Array_1[str], cMaxNames: int, pcNames: clr.Reference[int]) -> None: ...
    @abc.abstractmethod
    def GetRefTypeInfo(self, hRef: int, ppTI: clr.Reference[ITypeInfo]) -> None: ...
    @abc.abstractmethod
    def GetRefTypeOfImplType(self, index: int, href: clr.Reference[int]) -> None: ...
    @abc.abstractmethod
    def GetTypeAttr(self, ppTypeAttr: clr.Reference[int]) -> None: ...
    @abc.abstractmethod
    def GetTypeComp(self, ppTComp: clr.Reference[ITypeComp]) -> None: ...
    @abc.abstractmethod
    def GetVarDesc(self, index: int, ppVarDesc: clr.Reference[int]) -> None: ...
    @abc.abstractmethod
    def Invoke(self, pvInstance: typing.Any, memid: int, wFlags: int, pDispParams: clr.Reference[DISPPARAMS], pVarResult: int, pExcepInfo: int, puArgErr: clr.Reference[int]) -> None: ...
    @abc.abstractmethod
    def ReleaseFuncDesc(self, pFuncDesc: int) -> None: ...
    @abc.abstractmethod
    def ReleaseTypeAttr(self, pTypeAttr: int) -> None: ...
    @abc.abstractmethod
    def ReleaseVarDesc(self, pVarDesc: int) -> None: ...


class ITypeInfo2(ITypeInfo, typing.Protocol):
    @abc.abstractmethod
    def AddressOfMember(self, memid: int, invKind: INVOKEKIND, ppv: clr.Reference[int]) -> None: ...
    @abc.abstractmethod
    def CreateInstance(self, pUnkOuter: typing.Any, riid: clr.Reference[Guid], ppvObj: clr.Reference[typing.Any]) -> None: ...
    @abc.abstractmethod
    def GetAllCustData(self, pCustData: int) -> None: ...
    @abc.abstractmethod
    def GetAllFuncCustData(self, index: int, pCustData: int) -> None: ...
    @abc.abstractmethod
    def GetAllImplTypeCustData(self, index: int, pCustData: int) -> None: ...
    @abc.abstractmethod
    def GetAllParamCustData(self, indexFunc: int, indexParam: int, pCustData: int) -> None: ...
    @abc.abstractmethod
    def GetAllVarCustData(self, index: int, pCustData: int) -> None: ...
    @abc.abstractmethod
    def GetContainingTypeLib(self, ppTLB: clr.Reference[ITypeLib], pIndex: clr.Reference[int]) -> None: ...
    @abc.abstractmethod
    def GetCustData(self, guid: clr.Reference[Guid], pVarVal: clr.Reference[typing.Any]) -> None: ...
    @abc.abstractmethod
    def GetDllEntry(self, memid: int, invKind: INVOKEKIND, pBstrDllName: int, pBstrName: int, pwOrdinal: int) -> None: ...
    @abc.abstractmethod
    def GetDocumentation(self, index: int, strName: clr.Reference[str], strDocString: clr.Reference[str], dwHelpContext: clr.Reference[int], strHelpFile: clr.Reference[str]) -> None: ...
    @abc.abstractmethod
    def GetDocumentation2(self, memid: int, pbstrHelpString: clr.Reference[str], pdwHelpStringContext: clr.Reference[int], pbstrHelpStringDll: clr.Reference[str]) -> None: ...
    @abc.abstractmethod
    def GetFuncCustData(self, index: int, guid: clr.Reference[Guid], pVarVal: clr.Reference[typing.Any]) -> None: ...
    @abc.abstractmethod
    def GetFuncDesc(self, index: int, ppFuncDesc: clr.Reference[int]) -> None: ...
    @abc.abstractmethod
    def GetFuncIndexOfMemId(self, memid: int, invKind: INVOKEKIND, pFuncIndex: clr.Reference[int]) -> None: ...
    @abc.abstractmethod
    def GetIDsOfNames(self, rgszNames: Array_1[str], cNames: int, pMemId: Array_1[int]) -> None: ...
    @abc.abstractmethod
    def GetImplTypeCustData(self, index: int, guid: clr.Reference[Guid], pVarVal: clr.Reference[typing.Any]) -> None: ...
    @abc.abstractmethod
    def GetImplTypeFlags(self, index: int, pImplTypeFlags: clr.Reference[IMPLTYPEFLAGS]) -> None: ...
    @abc.abstractmethod
    def GetMops(self, memid: int, pBstrMops: clr.Reference[str]) -> None: ...
    @abc.abstractmethod
    def GetNames(self, memid: int, rgBstrNames: Array_1[str], cMaxNames: int, pcNames: clr.Reference[int]) -> None: ...
    @abc.abstractmethod
    def GetParamCustData(self, indexFunc: int, indexParam: int, guid: clr.Reference[Guid], pVarVal: clr.Reference[typing.Any]) -> None: ...
    @abc.abstractmethod
    def GetRefTypeInfo(self, hRef: int, ppTI: clr.Reference[ITypeInfo]) -> None: ...
    @abc.abstractmethod
    def GetRefTypeOfImplType(self, index: int, href: clr.Reference[int]) -> None: ...
    @abc.abstractmethod
    def GetTypeAttr(self, ppTypeAttr: clr.Reference[int]) -> None: ...
    @abc.abstractmethod
    def GetTypeComp(self, ppTComp: clr.Reference[ITypeComp]) -> None: ...
    @abc.abstractmethod
    def GetTypeFlags(self, pTypeFlags: clr.Reference[int]) -> None: ...
    @abc.abstractmethod
    def GetTypeKind(self, pTypeKind: clr.Reference[TYPEKIND]) -> None: ...
    @abc.abstractmethod
    def GetVarCustData(self, index: int, guid: clr.Reference[Guid], pVarVal: clr.Reference[typing.Any]) -> None: ...
    @abc.abstractmethod
    def GetVarDesc(self, index: int, ppVarDesc: clr.Reference[int]) -> None: ...
    @abc.abstractmethod
    def GetVarIndexOfMemId(self, memid: int, pVarIndex: clr.Reference[int]) -> None: ...
    @abc.abstractmethod
    def Invoke(self, pvInstance: typing.Any, memid: int, wFlags: int, pDispParams: clr.Reference[DISPPARAMS], pVarResult: int, pExcepInfo: int, puArgErr: clr.Reference[int]) -> None: ...
    @abc.abstractmethod
    def ReleaseFuncDesc(self, pFuncDesc: int) -> None: ...
    @abc.abstractmethod
    def ReleaseTypeAttr(self, pTypeAttr: int) -> None: ...
    @abc.abstractmethod
    def ReleaseVarDesc(self, pVarDesc: int) -> None: ...


class ITypeLib(typing.Protocol):
    @abc.abstractmethod
    def FindName(self, szNameBuf: str, lHashVal: int, ppTInfo: Array_1[ITypeInfo], rgMemId: Array_1[int], pcFound: clr.Reference[int]) -> None: ...
    @abc.abstractmethod
    def GetDocumentation(self, index: int, strName: clr.Reference[str], strDocString: clr.Reference[str], dwHelpContext: clr.Reference[int], strHelpFile: clr.Reference[str]) -> None: ...
    @abc.abstractmethod
    def GetLibAttr(self, ppTLibAttr: clr.Reference[int]) -> None: ...
    @abc.abstractmethod
    def GetTypeComp(self, ppTComp: clr.Reference[ITypeComp]) -> None: ...
    @abc.abstractmethod
    def GetTypeInfo(self, index: int, ppTI: clr.Reference[ITypeInfo]) -> None: ...
    @abc.abstractmethod
    def GetTypeInfoCount(self) -> int: ...
    @abc.abstractmethod
    def GetTypeInfoOfGuid(self, guid: clr.Reference[Guid], ppTInfo: clr.Reference[ITypeInfo]) -> None: ...
    @abc.abstractmethod
    def GetTypeInfoType(self, index: int, pTKind: clr.Reference[TYPEKIND]) -> None: ...
    @abc.abstractmethod
    def IsName(self, szNameBuf: str, lHashVal: int) -> bool: ...
    @abc.abstractmethod
    def ReleaseTLibAttr(self, pTLibAttr: int) -> None: ...


class ITypeLib2(ITypeLib, typing.Protocol):
    @abc.abstractmethod
    def FindName(self, szNameBuf: str, lHashVal: int, ppTInfo: Array_1[ITypeInfo], rgMemId: Array_1[int], pcFound: clr.Reference[int]) -> None: ...
    @abc.abstractmethod
    def GetAllCustData(self, pCustData: int) -> None: ...
    @abc.abstractmethod
    def GetCustData(self, guid: clr.Reference[Guid], pVarVal: clr.Reference[typing.Any]) -> None: ...
    @abc.abstractmethod
    def GetDocumentation(self, index: int, strName: clr.Reference[str], strDocString: clr.Reference[str], dwHelpContext: clr.Reference[int], strHelpFile: clr.Reference[str]) -> None: ...
    @abc.abstractmethod
    def GetDocumentation2(self, index: int, pbstrHelpString: clr.Reference[str], pdwHelpStringContext: clr.Reference[int], pbstrHelpStringDll: clr.Reference[str]) -> None: ...
    @abc.abstractmethod
    def GetLibAttr(self, ppTLibAttr: clr.Reference[int]) -> None: ...
    @abc.abstractmethod
    def GetLibStatistics(self, pcUniqueNames: int, pcchUniqueNames: clr.Reference[int]) -> None: ...
    @abc.abstractmethod
    def GetTypeComp(self, ppTComp: clr.Reference[ITypeComp]) -> None: ...
    @abc.abstractmethod
    def GetTypeInfo(self, index: int, ppTI: clr.Reference[ITypeInfo]) -> None: ...
    @abc.abstractmethod
    def GetTypeInfoCount(self) -> int: ...
    @abc.abstractmethod
    def GetTypeInfoOfGuid(self, guid: clr.Reference[Guid], ppTInfo: clr.Reference[ITypeInfo]) -> None: ...
    @abc.abstractmethod
    def GetTypeInfoType(self, index: int, pTKind: clr.Reference[TYPEKIND]) -> None: ...
    @abc.abstractmethod
    def IsName(self, szNameBuf: str, lHashVal: int) -> bool: ...
    @abc.abstractmethod
    def ReleaseTLibAttr(self, pTLibAttr: int) -> None: ...


class LIBFLAGS(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    LIBFLAG_FRESTRICTED : LIBFLAGS # 1
    LIBFLAG_FCONTROL : LIBFLAGS # 2
    LIBFLAG_FHIDDEN : LIBFLAGS # 4
    LIBFLAG_FHASDISKIMAGE : LIBFLAGS # 8


class PARAMDESC:
    lpVarValue : int
    wParamFlags : PARAMFLAG


class PARAMFLAG(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    PARAMFLAG_NONE : PARAMFLAG # 0
    PARAMFLAG_FIN : PARAMFLAG # 1
    PARAMFLAG_FOUT : PARAMFLAG # 2
    PARAMFLAG_FLCID : PARAMFLAG # 4
    PARAMFLAG_FRETVAL : PARAMFLAG # 8
    PARAMFLAG_FOPT : PARAMFLAG # 16
    PARAMFLAG_FHASDEFAULT : PARAMFLAG # 32
    PARAMFLAG_FHASCUSTDATA : PARAMFLAG # 64


class STATSTG:
    atime : FILETIME
    cbSize : int
    clsid : Guid
    ctime : FILETIME
    grfLocksSupported : int
    grfMode : int
    grfStateBits : int
    mtime : FILETIME
    pwcsName : str
    reserved : int
    type : int


class SYSKIND(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    SYS_WIN16 : SYSKIND # 0
    SYS_WIN32 : SYSKIND # 1
    SYS_MAC : SYSKIND # 2
    SYS_WIN64 : SYSKIND # 3


class TYPEATTR:
    cbAlignment : int
    cbSizeInstance : int
    cbSizeVft : int
    cFuncs : int
    cImplTypes : int
    cVars : int
    dwReserved : int
    guid : Guid
    idldescType : IDLDESC
    lcid : int
    lpstrSchema : int
    MEMBER_ID_NIL : int
    memidConstructor : int
    memidDestructor : int
    tdescAlias : TYPEDESC
    typekind : TYPEKIND
    wMajorVerNum : int
    wMinorVerNum : int
    wTypeFlags : TYPEFLAGS


class TYPEDESC:
    lpValue : int
    vt : int


class TYPEFLAGS(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    TYPEFLAG_FAPPOBJECT : TYPEFLAGS # 1
    TYPEFLAG_FCANCREATE : TYPEFLAGS # 2
    TYPEFLAG_FLICENSED : TYPEFLAGS # 4
    TYPEFLAG_FPREDECLID : TYPEFLAGS # 8
    TYPEFLAG_FHIDDEN : TYPEFLAGS # 16
    TYPEFLAG_FCONTROL : TYPEFLAGS # 32
    TYPEFLAG_FDUAL : TYPEFLAGS # 64
    TYPEFLAG_FNONEXTENSIBLE : TYPEFLAGS # 128
    TYPEFLAG_FOLEAUTOMATION : TYPEFLAGS # 256
    TYPEFLAG_FRESTRICTED : TYPEFLAGS # 512
    TYPEFLAG_FAGGREGATABLE : TYPEFLAGS # 1024
    TYPEFLAG_FREPLACEABLE : TYPEFLAGS # 2048
    TYPEFLAG_FDISPATCHABLE : TYPEFLAGS # 4096
    TYPEFLAG_FREVERSEBIND : TYPEFLAGS # 8192
    TYPEFLAG_FPROXY : TYPEFLAGS # 16384


class TYPEKIND(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    TKIND_ENUM : TYPEKIND # 0
    TKIND_RECORD : TYPEKIND # 1
    TKIND_MODULE : TYPEKIND # 2
    TKIND_INTERFACE : TYPEKIND # 3
    TKIND_DISPATCH : TYPEKIND # 4
    TKIND_COCLASS : TYPEKIND # 5
    TKIND_ALIAS : TYPEKIND # 6
    TKIND_UNION : TYPEKIND # 7
    TKIND_MAX : TYPEKIND # 8


class TYPELIBATTR:
    guid : Guid
    lcid : int
    syskind : SYSKIND
    wLibFlags : LIBFLAGS
    wMajorVerNum : int
    wMinorVerNum : int


class VARDESC:
    desc : VARDESC.DESCUNION
    elemdescVar : ELEMDESC
    lpstrSchema : str
    memid : int
    varkind : VARKIND
    wVarFlags : int

    class DESCUNION:
        lpvarValue : int
        oInst : int



class VARFLAGS(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    VARFLAG_FREADONLY : VARFLAGS # 1
    VARFLAG_FSOURCE : VARFLAGS # 2
    VARFLAG_FBINDABLE : VARFLAGS # 4
    VARFLAG_FREQUESTEDIT : VARFLAGS # 8
    VARFLAG_FDISPLAYBIND : VARFLAGS # 16
    VARFLAG_FDEFAULTBIND : VARFLAGS # 32
    VARFLAG_FHIDDEN : VARFLAGS # 64
    VARFLAG_FRESTRICTED : VARFLAGS # 128
    VARFLAG_FDEFAULTCOLLELEM : VARFLAGS # 256
    VARFLAG_FUIDEFAULT : VARFLAGS # 512
    VARFLAG_FNONBROWSABLE : VARFLAGS # 1024
    VARFLAG_FREPLACEABLE : VARFLAGS # 2048
    VARFLAG_FIMMEDIATEBIND : VARFLAGS # 4096


class VARKIND(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    VAR_PERINSTANCE : VARKIND # 0
    VAR_STATIC : VARKIND # 1
    VAR_CONST : VARKIND # 2
    VAR_DISPATCH : VARKIND # 3




from __future__ import annotations
from typing import Union
from itertools import product

import numpy as np
import pandas as pd
from SIMULTAN.Data.Components import SimDoubleParameter, SimParameterOperations
from SIMULTAN.Data.MultiValues import (SimMultiValueField3D, SimMultiValueField3DParameterSource, SimMultiValueBigTable,
                                       SimMultiValueBigTableHeader, SimMultiValueBigTableParameterSource)
from SIMULTAN.Data.Taxonomy import SimTaxonomyEntry
from System import Array, Double, Object, Boolean, String
from System.Collections.Generic import List as NetList
from System.Collections.Generic import ICollection as NetICollection
from pandas import DataFrame

import System


def numpy_to_simultan_multi_value_field_3d(array: np.ndarray,
                                           name: str = 'UnnamedField',
                                           x_axis: list = None,
                                           y_axis: list = None,
                                           z_axis: list = None,
                                           unit_x: str = '',
                                           unit_y: str = '',
                                           unit_z: str = '',
                                           can_interpolate: bool = True) -> SimMultiValueField3D:
    """
    Convert a numpy array to a SimMultiValueField3D object. The x, y, and z axes can be provided as lists.
    If not provided, the axes will be generated based on the shape of the input array.
    :param array: The input numpy array
    :param name: The name of the field, default is 'UnnamedField'
    :param x_axis: The x-axis of the field, default is None
    :param y_axis: The y-axis of the field, default is None
    :param z_axis: The z-axis of the field, default is None
    :param unit_x: The unit of the x-axis, default is ''
    :param unit_y: The unit of the y-axis, default is ''
    :param unit_z: The unit of the z-axis, default is ''
    :param can_interpolate: If True, the field can be interpolated. Default is True.
    data_model = None
    :return:
    """
    if not isinstance(array, np.ndarray):
        raise ValueError('The input array must be a numpy array.')

    if array.ndim < 3:
        if array.ndim == 1:
            array = np.expand_dims(array, axis=(1, 2))
        elif array.ndim == 2:
            array = np.expand_dims(array, axis=2)
        else:
            raise ValueError('The input array must be at least 2-dimensional.')

    if x_axis is None:
        x_axis = NetList[Double](Array[Double](list(range(array.astype(float).shape[0]))))

    if y_axis is None:
        y_axis = NetList[Double](Array[Double](list(range(array.astype(float).shape[1]))))

    if z_axis is None:
        z_axis = NetList[Double](Array[Double](list(range(array.astype(float).shape[2]))))

    data = NetList[Double](Array[Double](array.astype(float).flatten(order='F')))

    field = SimMultiValueField3D(name, x_axis, unit_x, y_axis, unit_y, z_axis, unit_z, data, can_interpolate)

    return field


def pandas_to_simultan_multi_value_big_table(df: pd.DataFrame,
                                             name: str = 'UnnamedField',
                                             unit_columns: str = '',
                                             unit_rows: str = '',
                                             column_headers: SimMultiValueBigTableHeader = None,
                                             row_headers: SimMultiValueBigTableHeader = None) -> SimMultiValueBigTable:
    """
    Convert a pandas DataFrame to a SimMultiValueField3D object. The x, y, and z axes can be provided as lists. If not
    provided, the axes will be generated based on the shape of the input DataFrame.
    :param df:
    :param name:
    :param unit_columns: The unit of the columns
    :param unit_rows: The unit of the rows
    :param column_headers: The column headers
    :param row_headers: The row headers
    :return: SimMultiValueBigTable
    """

    if not isinstance(df, DataFrame):
        raise ValueError('The input df must be a pandas DataFrame.')

    if column_headers is None:
        column_headers = NetICollection[SimMultiValueBigTableHeader](
            Array[SimMultiValueBigTableHeader]([SimMultiValueBigTableHeader(str(x), '') for x in df.columns]))

    if row_headers is None:
        row_headers = NetICollection[SimMultiValueBigTableHeader](
            Array[SimMultiValueBigTableHeader]([SimMultiValueBigTableHeader(str(x), '') for x in df.index]))

    #
    data = NetList[NetList[Double]]()
    for i in range(len(df.index)):
        data.Add(NetList[Double](Array[Double](df.values[i].astype(float))))

    field = SimMultiValueBigTable(name,
                                  unit_columns,
                                  unit_rows,
                                  column_headers,
                                  row_headers,
                                  data,
                                  True)
    return field


def simultan_multi_value_big_table_to_pandas(field: SimMultiValueBigTable) -> pd.DataFrame:
    """
    Convert a SimMultiValueBigTable object to a pandas DataFrame.
    :param field:
    :return:
    """
    if not isinstance(field, SimMultiValueBigTable):
        raise ValueError('The input field must be a SimMultiValueBigTable object.')

    data = np.ndarray((field.RowHeaders.Count, field.ColumnHeaders.Count))
    for i in range(field.RowHeaders.Count):
        data[i, :] = list(field.GetRow(i))

    df = pd.DataFrame(data, columns=[x.Name for x in field.ColumnHeaders], index=[x.Name for x in field.RowHeaders])

    return df


def simultan_multi_value_field_3d_to_numpy(field: SimMultiValueField3D, assert_ordered=False) -> np.ndarray:
    """
    Convert a SimMultiValueField3D object to a numpy array.
    :param field:
    :param assert_ordered: If True, the function will check if the field is ordered. Default is True.
    :return:
    """
    if not isinstance(field, SimMultiValueField3D):
        raise ValueError('The input field must be a SimMultiValueField3D object.')

    if assert_ordered:
        data = [x.Value for x in field.Field]
        x_axis = list(field.XAxis)
        y_axis = list(field.YAxis)
        z_axis = list(field.ZAxis)
        array = np.array(data).reshape(len(x_axis), len(y_axis), len(z_axis))
    else:
        array = np.empty((len(field.XAxis), len(field.YAxis), len(field.ZAxis)))
        for comb in product(range(len(field.XAxis)), range(len(field.YAxis)), range(len(field.ZAxis)), repeat=1):
            array[comb[0], comb[1], comb[2]] = field[comb[0], comb[1], comb[2]]

    return np.squeeze(array)


def add_field_to_data_model(field: SimMultiValueField3D, data_model: 'SimultanObject') -> SimMultiValueField3D:
    """
    Add a SimMultiValueField3D object to a data model.
    :param field: The SimMultiValueField3D object
    :param data_model: The data model
    :return: SimMultiValueField3D
    """
    if not isinstance(field, SimMultiValueField3D):
        raise ValueError('The input field must be a SimMultiValueField3D object.')

    if data_model is None:
        raise ValueError('The data model must not be None.')
    data_model.add_field(field)


def set_parameter_to_value_field(parameter: SimDoubleParameter,
                                 value: Union[SimMultiValueField3D, np.ndarray, DataFrame],
                                 field_name: str = None,
                                 x_ax_val: float = None,
                                 y_ax_val: float = None,
                                 z_ax_val: float = None,
                                 data_model=None) -> SimDoubleParameter:
    """
    Set the value of a field parameter to a SimMultiValueField3D object. The value can be a SimMultiValueField3D object,
    a numpy array, or a DataFrame.
    :param parameter: The field parameter
    :param value: The value of the parameter (SimMultiValueField3D, numpy array, or DataFrame)
    :param field_name: The name of the field, default is None
    :param x_ax_val: The value on the X-Axis (NOT the index/position)
    :param y_ax_val: The value on the Y-Axis (NOT the index/position)
    :param z_ax_val: The value on the Z-Axis (NOT the index/position)
    :param data_model: The data model, default is None. If data_model is not None, the SimMultiValueField3D will be
    added to the data model (if SimMultiValueField3D is created).
    :return: SimDoubleParameter
    """

    _ = x_ax_val

    def default_value_source(val_field: SimMultiValueField3D,
                             x_ax: float = None,
                             y_ax: float = None,
                             z_ax: float = None):
        x_ax = x_ax if x_ax is not None else val_field.XAxis[0]
        y_ax = y_ax if y_ax is not None else val_field.YAxis[0]
        z_ax = z_ax if z_ax is not None else val_field.ZAxis[0]
        return SimMultiValueField3DParameterSource(val_field, x_ax, y_ax, z_ax)

    if not isinstance(parameter, SimDoubleParameter):
        raise ValueError('The input parameter must be a SimDoubleParameter object.')

    if isinstance(value, SimMultiValueField3D):
        # source = value
        source = default_value_source(value, x_ax_val, y_ax_val, z_ax_val)
    elif isinstance(value, np.ndarray):
        value_field = numpy_to_simultan_multi_value_field_3d(value,
                                                             name=field_name if field_name is not None else str(
                                                                 parameter.Id) + '_field')
        if data_model is not None:
            data_model.add_field(value_field)

        # x_ax_val = x_ax_val if x_ax_val is not None else value_field.XAxis[0]
        # y_ax_val = y_ax_val if y_ax_val is not None else value_field.YAxis[0]
        # z_ax_val = z_ax_val if z_ax_val is not None else value_field.ZAxis[0]
        #
        # source = SimMultiValueField3DParameterSource(value_field, x_ax_val, y_ax_val, z_ax_val)
        source = default_value_source(value_field, x_ax_val, y_ax_val, z_ax_val)

    elif isinstance(value, pd.DataFrame):
        value_field = pandas_to_simultan_multi_value_big_table(value,
                                                               name=field_name if field_name is not None else str(
                                                                   parameter.Id) + '_field')
        if data_model is not None:
            data_model.add_field(value_field)

        source = SimMultiValueBigTableParameterSource(value_field, 0, 0)

    else:
        raise ValueError('The value of the field parameter must be a numpy array or a DataFrame.')

    parameter.set_ValueSource(source)
    return parameter


def create_field_parameter(value: Union[SimMultiValueField3D, np.ndarray, DataFrame],
                           name: str = 'unnamed parameter',
                           field_name: str = None,
                           unit: str = '',
                           slot: SimTaxonomyEntry = None,
                           min_value: float = None,
                           max_value: float = None,
                           allowed_operations: int = SimParameterOperations(0).All,
                           x_ax_val: float = None,
                           y_ax_val: float = None,
                           z_ax_val: float = None,
                           data_model=None
                           ) -> SimDoubleParameter:
    """
    Create a field parameter with a SimMultiValueField3D as value.  The value can be a SimMultiValueField3D object, a
    numpy array, or a DataFrame.
    :param value: The value of the parameter (SimMultiValueField3D, numpy array, or DataFrame)
    :param name: The name of the parameter, default is 'unnamed parameter'
    :param field_name: The name of the field, default is None
    :param unit: The unit of the parameter, default is ''
    :param slot: SimTaxonomyEntry
    :param min_value: The minimum value for the parameter, default is None
    :param max_value: The maximum value for the parameter, default is None
    :param allowed_operations: The allowed operations for the parameter (e.g. SimParameterOperations(0).All)
    :param x_ax_val: The value on the X-Axis (NOT the index/position)
    :param y_ax_val: The value on the Y-Axis (NOT the index/position)
    :param z_ax_val: The value on the Z-Axis (NOT the index/position)
    :param data_model: The data model, default is None. If data_model is not None, the SimMultiValueField3D will be
    added to the data model.
    :return: SimDoubleParameter
    """

    from .utils import create_sim_double_parameter

    param_init_dict = {'value': 0}

    for key in ['name', 'unit', 'slot', 'min_value', 'max_value', 'allowed_operations']:
        if locals()[key] is not None:
            param_init_dict[key] = locals()[key]

    param = create_sim_double_parameter(**param_init_dict)

    set_parameter_to_value_field(param,
                                 value,
                                 field_name=field_name,
                                 x_ax_val=x_ax_val,
                                 y_ax_val=y_ax_val,
                                 z_ax_val=z_ax_val,
                                 data_model=data_model
                                 )

    return param



import typing, abc

class IIdentity(typing.Protocol):
    @property
    def AuthenticationType(self) -> str: ...
    @property
    def IsAuthenticated(self) -> bool: ...
    @property
    def Name(self) -> str: ...


class IPrincipal(typing.Protocol):
    @property
    def Identity(self) -> IIdentity: ...
    @abc.abstractmethod
    def IsInRole(self, role: str) -> bool: ...


class PrincipalPolicy(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    UnauthenticatedPrincipal : PrincipalPolicy # 0
    NoPrincipal : PrincipalPolicy # 1
    WindowsPrincipal : PrincipalPolicy # 2


class TokenImpersonationLevel(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    None_ : TokenImpersonationLevel # 0
    Anonymous : TokenImpersonationLevel # 1
    Identification : TokenImpersonationLevel # 2
    Impersonation : TokenImpersonationLevel # 3
    Delegation : TokenImpersonationLevel # 4




import typing, abc
from SIMULTAN.Data.Geometry import GeometryModel, BaseGeometry, Vertex
from SIMULTAN.Data.SimNetworks import SimNetwork, ISimNetworkElement
from SIMULTAN.Exchange import ComponentGeometryExchange
from System.Collections.Generic import List_1, IEnumerable_1

class BaseSimnetworkGeometryConnector(abc.ABC):
    def Dispose(self) -> None: ...


class SimNetworkGeometryModelConnector:
    def __init__(self, model: GeometryModel, network: SimNetwork, exchange: ComponentGeometryExchange) -> None: ...
    def EndPartialTransform(self, effectedGeoms: List_1[BaseGeometry]) -> None: ...
    def GetNetworkElements(self, geometry: BaseGeometry) -> IEnumerable_1[ISimNetworkElement]: ...
    def GetStaticGroupGeometries(self, geom: BaseGeometry) -> IEnumerable_1[BaseGeometry]: ...
    def IstAssociatedWithStaticBlock(self, geom: BaseGeometry) -> bool: ...
    def StartMoveRotatePartialOperation(self, geom: BaseGeometry) -> IEnumerable_1[BaseGeometry]: ...
    def UpdateNetwork(self, network: SimNetwork) -> None: ...
    def UpdateStaticConnectors(self, geoms: List_1[Vertex]) -> None: ...


class ValidationError(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    RotationError : ValidationError # 0
    DistanceError : ValidationError # 1




import typing, abc
from System import Action_1, Exception

class IValueTaskSource_GenericClasses(abc.ABCMeta):
    Generic_IValueTaskSource_GenericClasses_IValueTaskSource_1_TResult = typing.TypeVar('Generic_IValueTaskSource_GenericClasses_IValueTaskSource_1_TResult')
    def __getitem__(self, types : typing.Type[Generic_IValueTaskSource_GenericClasses_IValueTaskSource_1_TResult]) -> typing.Type[IValueTaskSource_1[Generic_IValueTaskSource_GenericClasses_IValueTaskSource_1_TResult]]: ...

class IValueTaskSource(IValueTaskSource_0, metaclass =IValueTaskSource_GenericClasses): ...

class IValueTaskSource_0(typing.Protocol):
    @abc.abstractmethod
    def GetResult(self, token: int) -> None: ...
    @abc.abstractmethod
    def GetStatus(self, token: int) -> ValueTaskSourceStatus: ...
    @abc.abstractmethod
    def OnCompleted(self, continuation: Action_1[typing.Any], state: typing.Any, token: int, flags: ValueTaskSourceOnCompletedFlags) -> None: ...


IValueTaskSource_1_TResult = typing.TypeVar('IValueTaskSource_1_TResult', covariant=True)
class IValueTaskSource_1(typing.Generic[IValueTaskSource_1_TResult], typing.Protocol):
    @abc.abstractmethod
    def GetResult(self, token: int) -> IValueTaskSource_1_TResult: ...
    @abc.abstractmethod
    def GetStatus(self, token: int) -> ValueTaskSourceStatus: ...
    @abc.abstractmethod
    def OnCompleted(self, continuation: Action_1[typing.Any], state: typing.Any, token: int, flags: ValueTaskSourceOnCompletedFlags) -> None: ...


class ManualResetValueTaskSourceCore_GenericClasses(abc.ABCMeta):
    Generic_ManualResetValueTaskSourceCore_GenericClasses_ManualResetValueTaskSourceCore_1_TResult = typing.TypeVar('Generic_ManualResetValueTaskSourceCore_GenericClasses_ManualResetValueTaskSourceCore_1_TResult')
    def __getitem__(self, types : typing.Type[Generic_ManualResetValueTaskSourceCore_GenericClasses_ManualResetValueTaskSourceCore_1_TResult]) -> typing.Type[ManualResetValueTaskSourceCore_1[Generic_ManualResetValueTaskSourceCore_GenericClasses_ManualResetValueTaskSourceCore_1_TResult]]: ...

ManualResetValueTaskSourceCore : ManualResetValueTaskSourceCore_GenericClasses

ManualResetValueTaskSourceCore_1_TResult = typing.TypeVar('ManualResetValueTaskSourceCore_1_TResult')
class ManualResetValueTaskSourceCore_1(typing.Generic[ManualResetValueTaskSourceCore_1_TResult]):
    @property
    def RunContinuationsAsynchronously(self) -> bool: ...
    @RunContinuationsAsynchronously.setter
    def RunContinuationsAsynchronously(self, value: bool) -> bool: ...
    @property
    def Version(self) -> int: ...
    def GetResult(self, token: int) -> ManualResetValueTaskSourceCore_1_TResult: ...
    def GetStatus(self, token: int) -> ValueTaskSourceStatus: ...
    def OnCompleted(self, continuation: Action_1[typing.Any], state: typing.Any, token: int, flags: ValueTaskSourceOnCompletedFlags) -> None: ...
    def Reset(self) -> None: ...
    def SetException(self, error: Exception) -> None: ...
    def SetResult(self, result: ManualResetValueTaskSourceCore_1_TResult) -> None: ...


class ValueTaskSourceOnCompletedFlags(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    None_ : ValueTaskSourceOnCompletedFlags # 0
    UseSchedulingContext : ValueTaskSourceOnCompletedFlags # 1
    FlowExecutionContext : ValueTaskSourceOnCompletedFlags # 2


class ValueTaskSourceStatus(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    Pending : ValueTaskSourceStatus # 0
    Succeeded : ValueTaskSourceStatus # 1
    Faulted : ValueTaskSourceStatus # 2
    Canceled : ValueTaskSourceStatus # 3

